@@ -1,11 +1,14 @@
 # -*- coding: utf-8 -*-
+from __future__ import division, absolute_import, print_function
 
 import sys, os, re
 
 # Check Sphinx version
 import sphinx
-if sphinx.__version__ < "0.5":
-    raise RuntimeError("Sphinx 0.5.dev or newer required")
+if sphinx.__version__ < "1.0.1":
+    raise RuntimeError("Sphinx 1.0.1 or newer required")
+
+needs_sphinx = '1.0'
 
 # -----------------------------------------------------------------------------
 # General configuration
@@ -18,14 +21,8 @@ sys.path.insert(0, os.path.abspath('../sphinxext'))
 
 extensions = ['sphinx.ext.autodoc', 'sphinx.ext.pngmath', 'numpydoc',
               'sphinx.ext.intersphinx', 'sphinx.ext.coverage',
-              'sphinx.ext.doctest',
-              'plot_directive']
-
-if sphinx.__version__ >= "0.7":
-    extensions.append('sphinx.ext.autosummary')
-else:
-    extensions.append('autosummary')
-    extensions.append('only_directives')
+              'sphinx.ext.doctest', 'sphinx.ext.autosummary',
+              'matplotlib.sphinxext.plot_directive']
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
@@ -33,9 +30,6 @@ templates_path = ['_templates']
 # The suffix of source filenames.
 source_suffix = '.rst'
 
-# The master toctree document.
-#master_doc = 'index'
-
 # General substitutions.
 project = 'NumPy'
 copyright = '2008-2009, The Scipy community'
@@ -49,7 +43,7 @@ version = re.sub(r'(\d+\.\d+)\.\d+(.*)', r'\1\2', numpy.__version__)
 version = re.sub(r'(\.dev\d+).*?$', r'\1', version)
 # The full version, including alpha/beta/rc tags.
 release = numpy.__version__
-print version, release
+print("%s %s" % (version, release))
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
@@ -86,62 +80,50 @@ pygments_style = 'sphinx'
 # HTML output
 # -----------------------------------------------------------------------------
 
-# The style sheet to use for HTML and HTML Help pages. A file of that name
-# must exist either in Sphinx' static/ path, or in one of the custom paths
-# given in html_static_path.
-html_style = 'scipy.css'
+themedir = os.path.join(os.pardir, 'scipy-sphinx-theme', '_theme')
+if not os.path.isdir(themedir):
+    raise RuntimeError("Get the scipy-sphinx-theme first, "
+                       "via git submodule init && git submodule update")
 
-# The name for this set of Sphinx documents.  If None, it defaults to
-# "<project> v<release> documentation".
-html_title = "%s v%s Manual (DRAFT)" % (project, version)
+html_theme = 'scipy'
+html_theme_path = [themedir]
 
-# The name of an image file (within the static path) to place at the top of
-# the sidebar.
-html_logo = 'scipyshiny_small.png'
+if 'scipyorg' in tags:
+    # Build for the scipy.org website
+    html_theme_options = {
+        "edit_link": True,
+        "sidebar": "right",
+        "scipy_org_logo": True,
+        "rootlinks": [("http://scipy.org/", "Scipy.org"),
+                      ("http://docs.scipy.org/", "Docs")]
+    }
+else:
+    # Default build
+    html_theme_options = {
+        "edit_link": False,
+        "sidebar": "left",
+        "scipy_org_logo": False,
+        "rootlinks": []
+    }
+    html_sidebars = {'index': 'indexsidebar.html'}
 
-# Add any paths that contain custom static files (such as style sheets) here,
-# relative to this directory. They are copied after the builtin static files,
-# so a file named "default.css" will overwrite the builtin "default.css".
-html_static_path = ['_static']
-
-# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
-# using the given strftime format.
-html_last_updated_fmt = '%b %d, %Y'
-
-# If true, SmartyPants will be used to convert quotes and dashes to
-# typographically correct entities.
-#html_use_smartypants = True
-
-# Custom sidebar templates, maps document names to template names.
-html_sidebars = {
-    'index': 'indexsidebar.html'
-}
-
-# Additional templates that should be rendered to pages, maps page names to
-# template names.
 html_additional_pages = {
     'index': 'indexcontent.html',
 }
 
-# If false, no module index is generated.
+html_title = "%s v%s Manual" % (project, version)
+html_static_path = ['_static']
+html_last_updated_fmt = '%b %d, %Y'
+
 html_use_modindex = True
+html_copy_source = False
+html_domain_indices = False
+html_file_suffix = '.html'
 
-# If true, the reST sources are included in the HTML build as _sources/<name>.
-#html_copy_source = True
-
-# If true, an OpenSearch description file will be output, and all pages will
-# contain a <link> tag referring to it.  The value of this option must be the
-# base URL from which the finished HTML is served.
-#html_use_opensearch = ''
-
-# If nonempty, this is the file name suffix for HTML files (e.g. ".html").
-#html_file_suffix = '.html'
-
-# Output file base name for HTML help builder.
 htmlhelp_basename = 'numpy'
 
-# Pngmath should try to align formulas properly
 pngmath_use_preview = True
+pngmath_dvipng_args = ['-gamma', '1.5', '-D', '96', '-bg', 'Transparent']
 
 
 # -----------------------------------------------------------------------------
@@ -202,6 +184,18 @@ latex_preamble = r'''
 latex_use_modindex = False
 
 
+# -----------------------------------------------------------------------------
+# Texinfo output
+# -----------------------------------------------------------------------------
+
+texinfo_documents = [
+  ("contents", 'numpy', 'Numpy Documentation', _stdauthor, 'Numpy',
+   "NumPy: array processing for numbers, strings, records, and objects.",
+   'Programming',
+   1),
+]
+
+
 # -----------------------------------------------------------------------------
 # Intersphinx configuration
 # -----------------------------------------------------------------------------
@@ -222,9 +216,8 @@ numpydoc_use_plots = True
 # Autosummary
 # -----------------------------------------------------------------------------
 
-if sphinx.__version__ >= "0.7": 
-    import glob
-    autosummary_generate = glob.glob("reference/*.rst")
+import glob
+autosummary_generate = glob.glob("reference/*.rst")
 
 # -----------------------------------------------------------------------------
 # Coverage checker
@@ -256,8 +249,7 @@ plot_formats = [('png', 100), 'pdf']
 import math
 phi = (math.sqrt(5) + 1)/2
 
-import matplotlib
-matplotlib.rcParams.update({
+plot_rcparams = {
     'font.size': 8,
     'axes.titlesize': 8,
     'axes.labelsize': 8,
@@ -271,4 +263,69 @@ matplotlib.rcParams.update({
     'figure.subplot.top': 0.85,
     'figure.subplot.wspace': 0.4,
     'text.usetex': False,
-})
+}
+
+
+# -----------------------------------------------------------------------------
+# Source code links
+# -----------------------------------------------------------------------------
+
+import inspect
+from os.path import relpath, dirname
+
+for name in ['sphinx.ext.linkcode', 'numpydoc.linkcode']:
+    try:
+        __import__(name)
+        extensions.append(name)
+        break
+    except ImportError:
+        pass
+else:
+    print("NOTE: linkcode extension not found -- no links to source generated")
+
+def linkcode_resolve(domain, info):
+    """
+    Determine the URL corresponding to Python object
+    """
+    if domain != 'py':
+        return None
+
+    modname = info['module']
+    fullname = info['fullname']
+
+    submod = sys.modules.get(modname)
+    if submod is None:
+        return None
+
+    obj = submod
+    for part in fullname.split('.'):
+        try:
+            obj = getattr(obj, part)
+        except:
+            return None
+
+    try:
+        fn = inspect.getsourcefile(obj)
+    except:
+        fn = None
+    if not fn:
+        return None
+
+    try:
+        source, lineno = inspect.findsource(obj)
+    except:
+        lineno = None
+
+    if lineno:
+        linespec = "#L%d" % (lineno + 1)
+    else:
+        linespec = ""
+
+    fn = relpath(fn, start=dirname(numpy.__file__))
+
+    if 'dev' in numpy.__version__:
+        return "http://github.com/numpy/numpy/blob/master/numpy/%s%s" % (
+           fn, linespec)
+    else:
+        return "http://github.com/numpy/numpy/blob/v%s/numpy/%s%s" % (
+           numpy.__version__, fn, linespec)
