@@ -1,3 +1,5 @@
+from __future__ import division, print_function
+
 import os
 import re
 import struct
@@ -11,11 +13,12 @@ sys.path.pop(0)
 Zero = "PyUFunc_Zero"
 One = "PyUFunc_One"
 None_ = "PyUFunc_None"
+ReorderableNone = "PyUFunc_ReorderableNone"
 
-# Sentinel value to specify that the loop for the given TypeDescription uses the
-# pointer to arrays as its func_data.
-UsesArraysAsData = object()
-
+# Sentinel value to specify using the full type description in the
+# function name
+class FullTypeDescr(object):
+    pass
 
 class TypeDescription(object):
     """Type signature for a ufunc.
@@ -24,17 +27,23 @@ class TypeDescription(object):
     ----------
     type : str
         Character representing the nominal type.
-    func_data : str or None or UsesArraysAsData, optional
+    func_data : str or None or FullTypeDescr, optional
         The string representing the expression to insert into the data array, if
         any.
     in_ : str or None, optional
         The typecode(s) of the inputs.
     out : str or None, optional
         The typecode(s) of the outputs.
+    astype : dict or None, optional
+        If astype['x'] is 'y', uses PyUFunc_x_x_As_y_y/PyUFunc_xx_x_As_yy_y
+        instead of PyUFunc_x_x/PyUFunc_xx_x.
     """
-    def __init__(self, type, f=None, in_=None, out=None):
+    def __init__(self, type, f=None, in_=None, out=None, astype=None):
         self.type = type
         self.func_data = f
+        if astype is None:
+            astype = {}
+        self.astype_dict = astype
         if in_ is not None:
             in_ = in_.replace('P', type)
         self.in_ = in_
@@ -49,8 +58,9 @@ class TypeDescription(object):
         if self.out is None:
             self.out = self.type * nout
         assert len(self.out) == nout
+        self.astype = self.astype_dict.get(self.type, None)
 
-_fdata_map = dict(f='npy_%sf', d='npy_%s', g='npy_%sl',
+_fdata_map = dict(e='npy_%sf', f='npy_%sf', d='npy_%s', g='npy_%sl',
                   F='nc_%sf', D='nc_%s', G='nc_%sl')
 def build_func_data(types, f):
     func_data = []
@@ -59,7 +69,7 @@ def build_func_data(types, f):
         func_data.append(d)
     return func_data
 
-def TD(types, f=None, in_=None, out=None):
+def TD(types, f=None, astype=None, in_=None, out=None):
     if f is not None:
         if isinstance(f, str):
             func_data = build_func_data(types, f)
@@ -78,7 +88,7 @@ def TD(types, f=None, in_=None, out=None):
         out = (None,) * len(types)
     tds = []
     for t, fd, i, o in zip(types, func_data, in_, out):
-        tds.append(TypeDescription(t, f=fd, in_=i, out=o))
+        tds.append(TypeDescription(t, f=fd, in_=i, out=o, astype=astype))
     return tds
 
 class Ufunc(object):
@@ -86,14 +96,13 @@ class Ufunc(object):
 
     Attributes
     ----------
-
-    nin: number of input arguments
-    nout: number of output arguments
-    identity: identity element for a two-argument function
-    docstring: docstring for the ufunc
-    type_descriptions: list of TypeDescription objects
+    nin : number of input arguments
+    nout : number of output arguments
+    identity : identity element for a two-argument function
+    docstring : docstring for the ufunc
+    type_descriptions : list of TypeDescription objects
     """
-    def __init__(self, nin, nout, identity, docstring,
+    def __init__(self, nin, nout, identity, docstring, typereso,
                  *type_descriptions):
         self.nin = nin
         self.nout = nout
@@ -101,6 +110,7 @@ class Ufunc(object):
             identity = None_
         self.identity = identity
         self.docstring = docstring
+        self.typereso = typereso
         self.type_descriptions = []
         for td in type_descriptions:
             self.type_descriptions.extend(td)
@@ -113,7 +123,7 @@ import string
 if sys.version_info[0] < 3:
     UPPER_TABLE = string.maketrans(string.ascii_lowercase, string.ascii_uppercase)
 else:
-    UPPER_TABLE = bytes.maketrans(bytes(string.ascii_lowercase, "ascii"), 
+    UPPER_TABLE = bytes.maketrans(bytes(string.ascii_lowercase, "ascii"),
             bytes(string.ascii_uppercase, "ascii"))
 
 def english_upper(s):
@@ -166,44 +176,52 @@ chartoname = {'?': 'bool',
               'L': 'ulong',
               'q': 'longlong',
               'Q': 'ulonglong',
+              'e': 'half',
               'f': 'float',
               'd': 'double',
               'g': 'longdouble',
               'F': 'cfloat',
               'D': 'cdouble',
               'G': 'clongdouble',
+              'M': 'datetime',
+              'm': 'timedelta',
               'O': 'OBJECT',
               # '.' is like 'O', but calls a method of the object instead
               # of a function
               'P': 'OBJECT',
               }
 
-all = '?bBhHiIlLqQfdgFDGO'
+all = '?bBhHiIlLqQefdgFDGOMm'
 O = 'O'
 P = 'P'
 ints = 'bBhHiIlLqQ'
+times = 'Mm'
+timedeltaonly = 'm'
 intsO = ints + O
 bints = '?' + ints
 bintsO = bints + O
-flts = 'fdg'
+flts = 'efdg'
 fltsO = flts + O
 fltsP = flts + P
 cmplx = 'FDG'
 cmplxO = cmplx + O
 cmplxP = cmplx + P
 inexact = flts + cmplx
+inexactvec = 'fd'
 noint = inexact+O
 nointP = inexact+P
-allP = bints+flts+cmplxP
+allP = bints+times+flts+cmplxP
 nobool = all[1:]
-noobj = all[:-2] 
-nobool_or_obj = all[1:-2]
+noobj = all[:-3]+all[-2:]
+nobool_or_obj = all[1:-3]+all[-2:]
+nobool_or_datetime = all[1:-2]+all[-1:]
 intflt = ints+flts
 intfltcmplx = ints+flts+cmplx
-nocmplx = bints+flts
+nocmplx = bints+times+flts
 nocmplxO = nocmplx+O
 nocmplxP = nocmplx+P
 notimes_or_obj = bints + inexact
+nodatetime_or_obj = bints + inexact
 
 # Find which code corresponds to int64.
 int64 = ''
@@ -214,458 +232,567 @@ for code in 'bhilq':
         uint64 = english_upper(code)
         break
 
+# This dictionary describes all the ufunc implementations, generating
+# all the function names and their corresponding ufunc signatures.  TD is
+# an object which expands a list of character codes into an array of
+# TypeDescriptions.
 defdict = {
-'add' :
+'add':
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.add'),
+          'PyUFunc_AdditionTypeResolver',
           TD(notimes_or_obj),
+          [TypeDescription('M', FullTypeDescr, 'Mm', 'M'),
+           TypeDescription('m', FullTypeDescr, 'mm', 'm'),
+           TypeDescription('M', FullTypeDescr, 'mM', 'M'),
+          ],
           TD(O, f='PyNumber_Add'),
           ),
-'subtract' :
-    Ufunc(2, 1, Zero,
+'subtract':
+    Ufunc(2, 1, None, # Zero is only a unit to the right, not the left
           docstrings.get('numpy.core.umath.subtract'),
+          'PyUFunc_SubtractionTypeResolver',
           TD(notimes_or_obj),
+          [TypeDescription('M', FullTypeDescr, 'Mm', 'M'),
+           TypeDescription('m', FullTypeDescr, 'mm', 'm'),
+           TypeDescription('M', FullTypeDescr, 'MM', 'm'),
+          ],
           TD(O, f='PyNumber_Subtract'),
           ),
-'multiply' :
+'multiply':
     Ufunc(2, 1, One,
           docstrings.get('numpy.core.umath.multiply'),
+          'PyUFunc_MultiplicationTypeResolver',
           TD(notimes_or_obj),
+          [TypeDescription('m', FullTypeDescr, 'mq', 'm'),
+           TypeDescription('m', FullTypeDescr, 'qm', 'm'),
+           TypeDescription('m', FullTypeDescr, 'md', 'm'),
+           TypeDescription('m', FullTypeDescr, 'dm', 'm'),
+          ],
           TD(O, f='PyNumber_Multiply'),
           ),
-'divide' :
-    Ufunc(2, 1, One,
+'divide':
+    Ufunc(2, 1, None, # One is only a unit to the right, not the left
           docstrings.get('numpy.core.umath.divide'),
+          'PyUFunc_DivisionTypeResolver',
           TD(intfltcmplx),
+          [TypeDescription('m', FullTypeDescr, 'mq', 'm'),
+           TypeDescription('m', FullTypeDescr, 'md', 'm'),
+           TypeDescription('m', FullTypeDescr, 'mm', 'd'),
+          ],
           TD(O, f='PyNumber_Divide'),
           ),
-'floor_divide' :
-    Ufunc(2, 1, One,
+'floor_divide':
+    Ufunc(2, 1, None, # One is only a unit to the right, not the left
           docstrings.get('numpy.core.umath.floor_divide'),
+          'PyUFunc_DivisionTypeResolver',
           TD(intfltcmplx),
+          [TypeDescription('m', FullTypeDescr, 'mq', 'm'),
+           TypeDescription('m', FullTypeDescr, 'md', 'm'),
+           #TypeDescription('m', FullTypeDescr, 'mm', 'd'),
+          ],
           TD(O, f='PyNumber_FloorDivide'),
           ),
-'true_divide' :
-    Ufunc(2, 1, One,
+'true_divide':
+    Ufunc(2, 1, None, # One is only a unit to the right, not the left
           docstrings.get('numpy.core.umath.true_divide'),
+          'PyUFunc_DivisionTypeResolver',
           TD('bBhH', out='d'),
           TD('iIlLqQ', out='d'),
           TD(flts+cmplx),
+          [TypeDescription('m', FullTypeDescr, 'mq', 'm'),
+           TypeDescription('m', FullTypeDescr, 'md', 'm'),
+           TypeDescription('m', FullTypeDescr, 'mm', 'd'),
+          ],
           TD(O, f='PyNumber_TrueDivide'),
           ),
-'conjugate' :
+'conjugate':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.conjugate'),
+          None,
           TD(ints+flts+cmplx),
           TD(P, f='conjugate'),
           ),
-'fmod' :
-    Ufunc(2, 1, Zero,
+'fmod':
+    Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.fmod'),
+          None,
           TD(ints),
-          TD(flts, f='fmod'),
+          TD(flts, f='fmod', astype={'e':'f'}),
           TD(P, f='fmod'),
           ),
-'square' :
+'square':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.square'),
+          None,
           TD(ints+inexact),
           TD(O, f='Py_square'),
           ),
-'reciprocal' :
+'reciprocal':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.reciprocal'),
+          None,
           TD(ints+inexact),
           TD(O, f='Py_reciprocal'),
           ),
-'ones_like' :
+# This is no longer used as numpy.ones_like, however it is
+# still used by some internal calls.
+'_ones_like':
     Ufunc(1, 1, None,
-          docstrings.get('numpy.core.umath.ones_like'),
+          docstrings.get('numpy.core.umath._ones_like'),
+          'PyUFunc_OnesLikeTypeResolver',
           TD(noobj),
           TD(O, f='Py_get_one'),
           ),
-'power' :
-    Ufunc(2, 1, One,
+'power':
+    Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.power'),
+          None,
           TD(ints),
-          TD(inexact, f='pow'),
+          TD(inexact, f='pow', astype={'e':'f'}),
           TD(O, f='npy_ObjectPower'),
           ),
-'absolute' :
+'absolute':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.absolute'),
-          TD(bints+flts),
+          'PyUFunc_AbsoluteTypeResolver',
+          TD(bints+flts+timedeltaonly),
           TD(cmplx, out=('f', 'd', 'g')),
           TD(O, f='PyNumber_Absolute'),
           ),
-'_arg' :
+'_arg':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath._arg'),
+          None,
           TD(cmplx, out=('f', 'd', 'g')),
           ),
-'negative' :
+'negative':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.negative'),
-          TD(bints+flts),
+          'PyUFunc_SimpleUnaryOperationTypeResolver',
+          TD(bints+flts+timedeltaonly),
           TD(cmplx, f='neg'),
           TD(O, f='PyNumber_Negative'),
           ),
-'sign' :
+'sign':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.sign'),
-          TD(nobool),
+          'PyUFunc_SimpleUnaryOperationTypeResolver',
+          TD(nobool_or_datetime),
           ),
-'greater' :
+'greater':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.greater'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
           TD(all, out='?'),
           ),
-'greater_equal' :
+'greater_equal':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.greater_equal'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
           TD(all, out='?'),
           ),
-'less' :
+'less':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.less'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
           TD(all, out='?'),
           ),
-'less_equal' :
+'less_equal':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.less_equal'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
           TD(all, out='?'),
           ),
-'equal' :
+'equal':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.equal'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
           TD(all, out='?'),
           ),
-'not_equal' :
+'not_equal':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.not_equal'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
           TD(all, out='?'),
           ),
-'logical_and' :
+'logical_and':
     Ufunc(2, 1, One,
           docstrings.get('numpy.core.umath.logical_and'),
-          TD(noobj, out='?'),
-          TD(P, f='logical_and'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
+          TD(nodatetime_or_obj, out='?'),
+          TD(O, f='npy_ObjectLogicalAnd'),
           ),
-'logical_not' :
+'logical_not':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.logical_not'),
-          TD(noobj, out='?'),
-          TD(P, f='logical_not'),
+          None,
+          TD(nodatetime_or_obj, out='?'),
+          TD(O, f='npy_ObjectLogicalNot'),
           ),
-'logical_or' :
+'logical_or':
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.logical_or'),
-          TD(noobj, out='?'),
-          TD(P, f='logical_or'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
+          TD(nodatetime_or_obj, out='?'),
+          TD(O, f='npy_ObjectLogicalOr'),
           ),
-'logical_xor' :
+'logical_xor':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.logical_xor'),
-          TD(noobj, out='?'),
+          'PyUFunc_SimpleBinaryComparisonTypeResolver',
+          TD(nodatetime_or_obj, out='?'),
           TD(P, f='logical_xor'),
           ),
-'maximum' :
-    Ufunc(2, 1, None,
+'maximum':
+    Ufunc(2, 1, ReorderableNone,
           docstrings.get('numpy.core.umath.maximum'),
+          'PyUFunc_SimpleBinaryOperationTypeResolver',
           TD(noobj),
           TD(O, f='npy_ObjectMax')
           ),
-'minimum' :
-    Ufunc(2, 1, None,
+'minimum':
+    Ufunc(2, 1, ReorderableNone,
           docstrings.get('numpy.core.umath.minimum'),
+          'PyUFunc_SimpleBinaryOperationTypeResolver',
           TD(noobj),
           TD(O, f='npy_ObjectMin')
           ),
-'fmax' :
-    Ufunc(2, 1, None,
+'fmax':
+    Ufunc(2, 1, ReorderableNone,
           docstrings.get('numpy.core.umath.fmax'),
+          'PyUFunc_SimpleBinaryOperationTypeResolver',
           TD(noobj),
           TD(O, f='npy_ObjectMax')
           ),
-'fmin' :
-    Ufunc(2, 1, None,
+'fmin':
+    Ufunc(2, 1, ReorderableNone,
           docstrings.get('numpy.core.umath.fmin'),
+          'PyUFunc_SimpleBinaryOperationTypeResolver',
           TD(noobj),
           TD(O, f='npy_ObjectMin')
           ),
-'logaddexp' :
+'logaddexp':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.logaddexp'),
-          TD(flts, f="logaddexp")
+          None,
+          TD(flts, f="logaddexp", astype={'e':'f'})
           ),
-'logaddexp2' :
+'logaddexp2':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.logaddexp2'),
-          TD(flts, f="logaddexp2")
+          None,
+          TD(flts, f="logaddexp2", astype={'e':'f'})
           ),
-# FIXME: decide if the times should have the bitwise operations.
-'bitwise_and' :
+'bitwise_and':
     Ufunc(2, 1, One,
           docstrings.get('numpy.core.umath.bitwise_and'),
+          None,
           TD(bints),
           TD(O, f='PyNumber_And'),
           ),
-'bitwise_or' :
+'bitwise_or':
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.bitwise_or'),
+          None,
           TD(bints),
           TD(O, f='PyNumber_Or'),
           ),
-'bitwise_xor' :
+'bitwise_xor':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.bitwise_xor'),
+          None,
           TD(bints),
           TD(O, f='PyNumber_Xor'),
           ),
-'invert' :
+'invert':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.invert'),
+          None,
           TD(bints),
           TD(O, f='PyNumber_Invert'),
           ),
-'left_shift' :
+'left_shift':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.left_shift'),
+          None,
           TD(ints),
           TD(O, f='PyNumber_Lshift'),
           ),
-'right_shift' :
+'right_shift':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.right_shift'),
+          None,
           TD(ints),
           TD(O, f='PyNumber_Rshift'),
           ),
-'degrees' :
+'degrees':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.degrees'),
-          TD(fltsP, f='degrees'),
+          None,
+          TD(fltsP, f='degrees', astype={'e':'f'}),
           ),
-'rad2deg' :
+'rad2deg':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.rad2deg'),
-          TD(fltsP, f='rad2deg'),
+          None,
+          TD(fltsP, f='rad2deg', astype={'e':'f'}),
           ),
-'radians' :
+'radians':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.radians'),
-          TD(fltsP, f='radians'),
+          None,
+          TD(fltsP, f='radians', astype={'e':'f'}),
           ),
-'deg2rad' :
+'deg2rad':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.deg2rad'),
-          TD(fltsP, f='deg2rad'),
+          None,
+          TD(fltsP, f='deg2rad', astype={'e':'f'}),
           ),
-'arccos' :
+'arccos':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arccos'),
-          TD(inexact, f='acos'),
+          None,
+          TD(inexact, f='acos', astype={'e':'f'}),
           TD(P, f='arccos'),
           ),
-'arccosh' :
+'arccosh':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arccosh'),
-          TD(inexact, f='acosh'),
+          None,
+          TD(inexact, f='acosh', astype={'e':'f'}),
           TD(P, f='arccosh'),
           ),
-'arcsin' :
+'arcsin':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arcsin'),
-          TD(inexact, f='asin'),
+          None,
+          TD(inexact, f='asin', astype={'e':'f'}),
           TD(P, f='arcsin'),
           ),
-'arcsinh' :
+'arcsinh':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arcsinh'),
-          TD(inexact, f='asinh'),
+          None,
+          TD(inexact, f='asinh', astype={'e':'f'}),
           TD(P, f='arcsinh'),
           ),
-'arctan' :
+'arctan':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arctan'),
-          TD(inexact, f='atan'),
+          None,
+          TD(inexact, f='atan', astype={'e':'f'}),
           TD(P, f='arctan'),
           ),
-'arctanh' :
+'arctanh':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arctanh'),
-          TD(inexact, f='atanh'),
+          None,
+          TD(inexact, f='atanh', astype={'e':'f'}),
           TD(P, f='arctanh'),
           ),
-'cos' :
+'cos':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.cos'),
-          TD(inexact, f='cos'),
+          None,
+          TD(inexact, f='cos', astype={'e':'f'}),
           TD(P, f='cos'),
           ),
-'sin' :
+'sin':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.sin'),
-          TD(inexact, f='sin'),
+          None,
+          TD(inexact, f='sin', astype={'e':'f'}),
           TD(P, f='sin'),
           ),
-'tan' :
+'tan':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.tan'),
-          TD(inexact, f='tan'),
+          None,
+          TD(inexact, f='tan', astype={'e':'f'}),
           TD(P, f='tan'),
           ),
-'cosh' :
+'cosh':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.cosh'),
-          TD(inexact, f='cosh'),
+          None,
+          TD(inexact, f='cosh', astype={'e':'f'}),
           TD(P, f='cosh'),
           ),
-'sinh' :
+'sinh':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.sinh'),
-          TD(inexact, f='sinh'),
+          None,
+          TD(inexact, f='sinh', astype={'e':'f'}),
           TD(P, f='sinh'),
           ),
-'tanh' :
+'tanh':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.tanh'),
-          TD(inexact, f='tanh'),
+          None,
+          TD(inexact, f='tanh', astype={'e':'f'}),
           TD(P, f='tanh'),
           ),
-'exp' :
+'exp':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.exp'),
-          TD(inexact, f='exp'),
+          None,
+          TD(inexact, f='exp', astype={'e':'f'}),
           TD(P, f='exp'),
           ),
-'exp2' :
+'exp2':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.exp2'),
-          TD(inexact, f='exp2'),
+          None,
+          TD(inexact, f='exp2', astype={'e':'f'}),
           TD(P, f='exp2'),
           ),
-'expm1' :
+'expm1':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.expm1'),
-          TD(inexact, f='expm1'),
+          None,
+          TD(inexact, f='expm1', astype={'e':'f'}),
           TD(P, f='expm1'),
           ),
-'log' :
+'log':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.log'),
-          TD(inexact, f='log'),
+          None,
+          TD(inexact, f='log', astype={'e':'f'}),
           TD(P, f='log'),
           ),
-'log2' :
+'log2':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.log2'),
-          TD(inexact, f='log2'),
+          None,
+          TD(inexact, f='log2', astype={'e':'f'}),
           TD(P, f='log2'),
           ),
-'log10' :
+'log10':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.log10'),
-          TD(inexact, f='log10'),
+          None,
+          TD(inexact, f='log10', astype={'e':'f'}),
           TD(P, f='log10'),
           ),
-'log1p' :
+'log1p':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.log1p'),
-          TD(inexact, f='log1p'),
+          None,
+          TD(inexact, f='log1p', astype={'e':'f'}),
           TD(P, f='log1p'),
           ),
-'sqrt' :
+'sqrt':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.sqrt'),
-          TD(inexact, f='sqrt'),
+          None,
+          TD(inexactvec),
+          TD(inexact, f='sqrt', astype={'e':'f'}),
           TD(P, f='sqrt'),
           ),
-'ceil' :
+'ceil':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.ceil'),
-          TD(flts, f='ceil'),
+          None,
+          TD(flts, f='ceil', astype={'e':'f'}),
           TD(P, f='ceil'),
           ),
-'trunc' :
+'trunc':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.trunc'),
-          TD(flts, f='trunc'),
+          None,
+          TD(flts, f='trunc', astype={'e':'f'}),
           TD(P, f='trunc'),
           ),
-'fabs' :
+'fabs':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.fabs'),
-          TD(flts, f='fabs'),
+          None,
+          TD(flts, f='fabs', astype={'e':'f'}),
           TD(P, f='fabs'),
        ),
-'floor' :
+'floor':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.floor'),
-          TD(flts, f='floor'),
+          None,
+          TD(flts, f='floor', astype={'e':'f'}),
           TD(P, f='floor'),
           ),
-'rint' :
+'rint':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.rint'),
-          TD(inexact, f='rint'),
+          None,
+          TD(inexact, f='rint', astype={'e':'f'}),
           TD(P, f='rint'),
           ),
-'arctan2' :
+'arctan2':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.arctan2'),
-          TD(flts, f='atan2'),
+          None,
+          TD(flts, f='atan2', astype={'e':'f'}),
           TD(P, f='arctan2'),
           ),
-'remainder' :
+'remainder':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.remainder'),
+          None,
           TD(intflt),
           TD(O, f='PyNumber_Remainder'),
           ),
-'hypot' :
+'hypot':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.hypot'),
-          TD(flts, f='hypot'),
+          None,
+          TD(flts, f='hypot', astype={'e':'f'}),
           TD(P, f='hypot'),
           ),
-'isnan' :
+'isnan':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.isnan'),
+          None,
           TD(inexact, out='?'),
           ),
-'isinf' :
+'isinf':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.isinf'),
+          None,
           TD(inexact, out='?'),
           ),
-'isfinite' :
+'isfinite':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.isfinite'),
+          None,
           TD(inexact, out='?'),
           ),
-'signbit' :
+'signbit':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.signbit'),
+          None,
           TD(flts, out='?'),
           ),
-'copysign' :
+'copysign':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.copysign'),
+          None,
           TD(flts),
           ),
-'nextafter' :
+'nextafter':
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.nextafter'),
+          None,
           TD(flts),
           ),
-'spacing' :
+'spacing':
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.spacing'),
+          None,
           TD(flts),
           ),
-'modf' :
+'modf':
     Ufunc(1, 2, None,
           docstrings.get('numpy.core.umath.modf'),
+          None,
           TD(flts),
           ),
 }
@@ -674,14 +801,15 @@ if sys.version_info[0] >= 3:
     # Will be aliased to true_divide in umathmodule.c.src:InitOtherOperators
     del defdict['divide']
 
-def indent(st,spaces):
+def indent(st, spaces):
     indention = ' '*spaces
-    indented = indention + st.replace('\n','\n'+indention)
+    indented = indention + st.replace('\n', '\n'+indention)
     # trim off any trailing spaces
-    indented = re.sub(r' +$',r'',indented)
+    indented = re.sub(r' +$', r'', indented)
     return indented
 
-chartotype1 = {'f': 'f_f',
+chartotype1 = {'e': 'e_e',
+               'f': 'f_f',
                'd': 'd_d',
                'g': 'g_g',
                'F': 'F_F',
@@ -690,7 +818,8 @@ chartotype1 = {'f': 'f_f',
                'O': 'O_O',
                'P': 'O_O_method'}
 
-chartotype2 = {'f': 'ff_f',
+chartotype2 = {'e': 'ee_e',
+               'f': 'ff_f',
                'd': 'dd_d',
                'g': 'gg_g',
                'F': 'FF_F',
@@ -709,8 +838,7 @@ def make_arrays(funcdict):
     #
     code1list = []
     code2list = []
-    names = list(funcdict.keys())
-    names.sort()
+    names = sorted(funcdict.keys())
     for name in names:
         uf = funcdict[name]
         funclist = []
@@ -726,10 +854,13 @@ def make_arrays(funcdict):
             thedict = chartotype1  # one input and one output
 
         for t in uf.type_descriptions:
-            if t.func_data not in (None, UsesArraysAsData):
+            if t.func_data not in (None, FullTypeDescr):
                 funclist.append('NULL')
-                astr = '%s_functions[%d] = PyUFunc_%s;' % \
-                       (name, k, thedict[t.type])
+                astype = ''
+                if not t.astype is None:
+                    astype = '_As_%s' % thedict[t.astype]
+                astr = '%s_functions[%d] = PyUFunc_%s%s;' % \
+                       (name, k, thedict[t.type], astype)
                 code2list.append(astr)
                 if t.type == 'O':
                     astr = '%s_data[%d] = (void *) %s;' % \
@@ -745,18 +876,17 @@ def make_arrays(funcdict):
                     datalist.append('(void *)NULL')
                     #datalist.append('(void *)%s' % t.func_data)
                 sub += 1
-            elif t.func_data is UsesArraysAsData:
+            elif t.func_data is FullTypeDescr:
                 tname = english_upper(chartoname[t.type])
                 datalist.append('(void *)NULL')
                 funclist.append('%s_%s_%s_%s' % (tname, t.in_, t.out, name))
-                code2list.append('PyUFunc_SetUsesArraysAsData(%s_data, %s);' % (name, k))
             else:
                 datalist.append('(void *)NULL')
                 tname = english_upper(chartoname[t.type])
                 funclist.append('%s_%s' % (tname, name))
 
             for x in t.in_ + t.out:
-                siglist.append('PyArray_%s' % (english_upper(chartoname[x]),))
+                siglist.append('NPY_%s' % (english_upper(chartoname[x]),))
 
             k += 1
 
@@ -769,12 +899,11 @@ def make_arrays(funcdict):
                          % (name, datanames))
         code1list.append("static char %s_signatures[] = { %s };" \
                          % (name, signames))
-    return "\n".join(code1list),"\n".join(code2list)
+    return "\n".join(code1list), "\n".join(code2list)
 
 def make_ufuncs(funcdict):
     code3list = []
-    names = list(funcdict.keys())
-    names.sort()
+    names = sorted(funcdict.keys())
     for name in names:
         uf = funcdict[name]
         mlist = []
@@ -800,17 +929,21 @@ r"""f = PyUFunc_FromFuncAndData(%s_functions, %s_data, %s_signatures, %d,
                                                 uf.nin, uf.nout,
                                                 uf.identity,
                                                 name, docstring))
+        if uf.typereso != None:
+            mlist.append(
+                r"((PyUFuncObject *)f)->type_resolver = &%s;" %
+                                                                uf.typereso)
         mlist.append(r"""PyDict_SetItemString(dictionary, "%s", f);""" % name)
         mlist.append(r"""Py_DECREF(f);""")
         code3list.append('\n'.join(mlist))
     return '\n'.join(code3list)
 
 
-def make_code(funcdict,filename):
+def make_code(funcdict, filename):
     code1, code2 = make_arrays(funcdict)
     code3 = make_ufuncs(funcdict)
-    code2 = indent(code2,4)
-    code3 = indent(code3,4)
+    code2 = indent(code2, 4)
+    code3 = indent(code3, 4)
     code = r"""
 
 /** Warning this file is autogenerated!!!
@@ -833,7 +966,7 @@ InitOperators(PyObject *dictionary) {
 
 if __name__ == "__main__":
     filename = __file__
-    fid = open('__umath_generated.c','w')
+    fid = open('__umath_generated.c', 'w')
     code = make_code(defdict, filename)
     fid.write(code)
     fid.close()
