@@ -18,6 +18,8 @@
 #ifndef _NPY_CPUARCH_H_
 #define _NPY_CPUARCH_H_
 
+#include "numpyconfig.h"
+
 #if defined( __i386__ ) || defined(i386) || defined(_M_IX86)
     /*
      * __i386__ is defined by gcc and Intel compiler on Linux,
@@ -32,10 +34,11 @@
      * _M_AMD64 defined by MS compiler
      */
     #define NPY_CPU_AMD64
-#elif defined(__ppc__) || defined(__powerpc__)
+#elif defined(__ppc__) || defined(__powerpc__) || defined(_ARCH_PPC)
     /*
      * __ppc__ is defined by gcc, I remember having seen __powerpc__ once,
      * but can't find it ATM
+     * _ARCH_PPC is used by at least gcc on AIX
      */
     #define NPY_CPU_PPC
 #elif defined(__ppc64__)
@@ -63,6 +66,10 @@
     #define NPY_CPU_MIPSEL
 #elif defined(__MIPSEB__)
     #define NPY_CPU_MIPSEB
+#elif defined(__aarch64__)
+    #define NPY_CPU_AARCH64
+#elif defined(__mc68000__)
+    #define NPY_CPU_M68K
 #else
     #error Unknown CPU, please report this to numpy maintainers with \
     information about your platform (OS, CPU and compiler)
@@ -95,6 +102,9 @@
             ((char*)(dst))[5] = ((char*)(src))[5]; \
             ((char*)(dst))[6] = ((char*)(src))[6]; \
             ((char*)(dst))[7] = ((char*)(src))[7];
+    #else
+        #error Unknown architecture, please report this to numpy maintainers with \
+        information about your platform (OS, CPU and compiler)
     #endif
 #endif
 
