@@ -34,15 +34,19 @@ Record arrays allow us to access fields as properties::
   array([ 2.,  2.])
 
 """
+from __future__ import division, absolute_import, print_function
+
+import sys
+import os
+
+from . import numeric as sb
+from .defchararray import chararray
+from . import numerictypes as nt
+from numpy.compat import isfileobj, bytes, long
+
 # All of the functions allow formats to be a dtype
 __all__ = ['record', 'recarray', 'format_parser']
 
-import numeric as sb
-from defchararray import chararray
-import numerictypes as nt
-import types
-import os
-import sys
 
 ndarray = sb.ndarray
 
@@ -124,7 +128,7 @@ class format_parser:
     >>> np.format_parser(['f8', 'i4', 'a5'], ['col1', 'col2', 'col3'],
     ...                  ['T1', 'T2', 'T3']).dtype
     dtype([(('T1', 'col1'), '<f8'), (('T2', 'col2'), '<i4'),
-           (('T3', 'col3'), '|S5')]
+           (('T3', 'col3'), '|S5')])
 
     `names` and/or `titles` can be empty lists. If `titles` is an empty list,
     titles will simply not appear. If `names` is empty, default field names
@@ -147,7 +151,7 @@ class format_parser:
         """ Parse the field formats """
 
         if formats is None:
-            raise ValueError, "Need formats argument"
+            raise ValueError("Need formats argument")
         if isinstance(formats, list):
             if len(formats) < 2:
                 formats.append('')
@@ -167,12 +171,12 @@ class format_parser:
         attribute """
 
         if (names):
-            if (type(names) in [types.ListType, types.TupleType]):
+            if (type(names) in [list, tuple]):
                 pass
-            elif (type(names) == types.StringType):
+            elif isinstance(names, str):
                 names = names.split(',')
             else:
-                raise NameError, "illegal input names %s" % `names`
+                raise NameError("illegal input names %s" % repr(names))
 
             self._names = [n.strip() for n in names[:self._nfields]]
         else:
@@ -187,7 +191,7 @@ class format_parser:
         # check for redundant names
         _dup = find_duplicate(self._names)
         if _dup:
-            raise ValueError, "Duplicate field names: %s" % _dup
+            raise ValueError("Duplicate field names: %s" % _dup)
 
         if (titles):
             self._titles = [n.strip() for n in titles[:self._nfields]]
@@ -242,13 +246,13 @@ class record(nt.void):
                 return obj.view(chararray)
             return obj
         else:
-            raise AttributeError, "'record' object has no "\
-                  "attribute '%s'" % attr
+            raise AttributeError("'record' object has no "
+                    "attribute '%s'" % attr)
 
 
     def __setattr__(self, attr, val):
         if attr in ['setfield', 'getfield', 'dtype']:
-            raise AttributeError, "Cannot set '%s' attribute" % attr
+            raise AttributeError("Cannot set '%s' attribute" % attr)
         fielddict = nt.void.__getattribute__(self, 'dtype').fields
         res = fielddict.get(attr, None)
         if res:
@@ -257,8 +261,8 @@ class record(nt.void):
             if getattr(self, attr, None):
                 return nt.void.__setattr__(self, attr, val)
             else:
-                raise AttributeError, "'record' object has no "\
-                      "attribute '%s'" % attr
+                raise AttributeError("'record' object has no "
+                        "attribute '%s'" % attr)
 
     def pprint(self):
         """Pretty-print all fields."""
@@ -283,7 +287,7 @@ class recarray(ndarray):
     """
     Construct an ndarray that allows field access using attributes.
 
-    Arrays may have a data-types containing fields, analagous
+    Arrays may have a data-types containing fields, analogous
     to columns in a spread sheet.  An example is ``[(x, int), (y, float)]``,
     where each entry in the array is a pair of ``(int, float)``.  Normally,
     these attributes are accessed using dictionary lookups such as ``arr['x']``
@@ -329,6 +333,8 @@ class recarray(ndarray):
         occupy in memory).
     offset : int, optional
         Start reading buffer (`buf`) from this offset onwards.
+    order : {'C', 'F'}, optional
+        Row-major or column-major order.
 
     Returns
     -------
@@ -344,7 +350,7 @@ class recarray(ndarray):
     Notes
     -----
     This constructor can be compared to ``empty``: it creates a new record
-    array but does not fill it with data.  To create a reccord array from data,
+    array but does not fill it with data.  To create a record array from data,
     use one of the following methods:
 
     1. Create a standard ndarray and convert it to a record array,
@@ -385,7 +391,7 @@ class recarray(ndarray):
     """
     def __new__(subtype, shape, dtype=None, buf=None, offset=0, strides=None,
                 formats=None, names=None, titles=None,
-                byteorder=None, aligned=False):
+                byteorder=None, aligned=False, order='C'):
 
         if dtype is not None:
             descr = sb.dtype(dtype)
@@ -393,11 +399,11 @@ class recarray(ndarray):
             descr = format_parser(formats, names, titles, aligned, byteorder)._descr
 
         if buf is None:
-            self = ndarray.__new__(subtype, shape, (record, descr))
+            self = ndarray.__new__(subtype, shape, (record, descr), order=order)
         else:
             self = ndarray.__new__(subtype, shape, (record, descr),
                                       buffer=buf, offset=offset,
-                                      strides=strides)
+                                      strides=strides, order=order)
         return self
 
     def __getattribute__(self, attr):
@@ -409,7 +415,7 @@ class recarray(ndarray):
         try:
             res = fielddict[attr][:2]
         except (TypeError, KeyError):
-            raise AttributeError, "record array has no attribute %s" % attr
+            raise AttributeError("record array has no attribute %s" % attr)
         obj = self.getfield(*res)
         # if it has fields return a recarray, otherwise return
         # normal array
@@ -432,7 +438,7 @@ class recarray(ndarray):
             fielddict = ndarray.__getattribute__(self, 'dtype').fields or {}
             if attr not in fielddict:
                 exctype, value = sys.exc_info()[:2]
-                raise exctype, value
+                raise exctype(value)
         else:
             fielddict = ndarray.__getattribute__(self, 'dtype').fields or {}
             if attr not in fielddict:
@@ -446,7 +452,7 @@ class recarray(ndarray):
         try:
             res = fielddict[attr][:2]
         except (TypeError, KeyError):
-            raise AttributeError, "record array has no attribute %s" % attr
+            raise AttributeError("record array has no attribute %s" % attr)
         return self.setfield(val, *res)
 
     def __getitem__(self, indx):
@@ -524,10 +530,10 @@ def fromarrays(arrayList, dtype=None, shape=None, formats=None,
         formats = ''
         for obj in arrayList:
             if not isinstance(obj, ndarray):
-                raise ValueError, "item in the array list must be an ndarray."
+                raise ValueError("item in the array list must be an ndarray.")
             formats += _typestr[obj.dtype.type]
             if issubclass(obj.dtype.type, nt.flexible):
-                formats += `obj.itemsize`
+                formats += repr(obj.itemsize)
             formats += ','
         formats = formats[:-1]
 
@@ -541,8 +547,8 @@ def fromarrays(arrayList, dtype=None, shape=None, formats=None,
 
     # Determine shape from data-type.
     if len(descr) != len(arrayList):
-        raise ValueError, "mismatch between the number of fields "\
-              "and the number of arrays"
+        raise ValueError("mismatch between the number of fields "
+                "and the number of arrays")
 
     d0 = descr[0].shape
     nn = len(d0)
@@ -553,7 +559,7 @@ def fromarrays(arrayList, dtype=None, shape=None, formats=None,
         nn = len(descr[k].shape)
         testshape = obj.shape[:len(obj.shape) - nn]
         if testshape != shape:
-            raise ValueError, "array-shape mismatch in array %d" % k
+            raise ValueError("array-shape mismatch in array %d" % k)
 
     _array = recarray(shape, descr)
 
@@ -589,15 +595,15 @@ def fromrecords(recList, dtype=None, shape=None, formats=None, names=None,
     >>> r.col2
     chararray(['dbe', 'de'],
           dtype='|S3')
-    >>> import cPickle
-    >>> print cPickle.loads(cPickle.dumps(r))
+    >>> import pickle
+    >>> print pickle.loads(pickle.dumps(r))
     [(456, 'dbe', 1.2) (2, 'de', 1.3)]
     """
 
     nfields = len(recList[0])
     if formats is None and dtype is None:  # slower
         obj = sb.array(recList, dtype=object)
-        arrlist = [sb.array(obj[..., i].tolist()) for i in xrange(nfields)]
+        arrlist = [sb.array(obj[..., i].tolist()) for i in range(nfields)]
         return fromarrays(arrlist, formats=formats, shape=shape, names=names,
                           titles=titles, aligned=aligned, byteorder=byteorder)
 
@@ -614,9 +620,9 @@ def fromrecords(recList, dtype=None, shape=None, formats=None, names=None,
         if isinstance(shape, (int, long)):
             shape = (shape,)
         if len(shape) > 1:
-            raise ValueError, "Can only deal with 1-d array."
+            raise ValueError("Can only deal with 1-d array.")
         _array = recarray(shape, descr)
-        for k in xrange(_array.size):
+        for k in range(_array.size):
             _array[k] = tuple(recList[k])
         return _array
     else:
@@ -635,7 +641,7 @@ def fromstring(datastring, dtype=None, shape=None, offset=0, formats=None,
 
 
     if dtype is None and formats is None:
-        raise ValueError, "Must have dtype= or formats="
+        raise ValueError("Must have dtype= or formats=")
 
     if dtype is not None:
         descr = sb.dtype(dtype)
@@ -731,7 +737,8 @@ def array(obj, dtype=None, shape=None, offset=0, strides=None, formats=None,
     """Construct a record array from a wide-variety of objects.
     """
 
-    if isinstance(obj, (type(None), str, file)) and (formats is None) \
+    if (isinstance(obj, (type(None), str)) or isfileobj(obj)) \
+           and (formats is None) \
            and (dtype is None):
         raise ValueError("Must define formats (or dtype) if object is "\
                          "None, string, or an open file")
@@ -754,7 +761,8 @@ def array(obj, dtype=None, shape=None, offset=0, strides=None, formats=None,
         if shape is None:
             raise ValueError("Must define a shape if obj is None")
         return recarray(shape, dtype, buf=obj, offset=offset, strides=strides)
-    elif isinstance(obj, str):
+
+    elif isinstance(obj, bytes):
         return fromstring(obj, dtype, shape=shape, offset=offset, **kwds)
 
     elif isinstance(obj, (list, tuple)):
@@ -772,7 +780,7 @@ def array(obj, dtype=None, shape=None, offset=0, strides=None, formats=None,
             new = new.copy()
         return new
 
-    elif isinstance(obj, file):
+    elif isfileobj(obj):
         return fromfile(obj, dtype=dtype, shape=shape, offset=offset)
 
     elif isinstance(obj, ndarray):
