@@ -5,4 +5,7 @@
 extern NPY_NO_EXPORT PyMethodDef array_methods[];
 #endif
 
+NPY_NO_EXPORT const char *
+npy_casting_to_string(NPY_CASTING casting);
+
 #endif
