@@ -33,7 +33,7 @@ NPY_NO_EXPORT void
 initialize_numeric_types(void);
 
 NPY_NO_EXPORT void
-format_longdouble(char *buf, size_t buflen, longdouble val, unsigned int prec);
+format_longdouble(char *buf, size_t buflen, npy_longdouble val, unsigned int prec);
 
 #if PY_VERSION_HEX >= 0x03000000
 NPY_NO_EXPORT void
