@@ -1,12 +1,11 @@
+from __future__ import division, absolute_import, print_function
+
 import sys
 from numpy.testing import *
-set_package_path()
 from swig_ext import example2
-restore_path()
 
-class TestExample2(NumpyTestCase):
-
-    def check_zoo(self):
+class TestExample2(TestCase):
+    def test_zoo(self):
         z = example2.Zoo()
         z.shut_up('Tiger')
         z.shut_up('Lion')
@@ -14,4 +13,4 @@ class TestExample2(NumpyTestCase):
 
 
 if __name__ == "__main__":
-    NumpyTest().run()
+    run_module_suite()
