@@ -7,9 +7,6 @@
 #include "numpy/ufuncobject.h"
 #include "string.h"
 
-#if (PY_VERSION_HEX < 0x02060000)
-#define Py_TYPE(o)    (((PyObject*)(o))->ob_type)
-#endif
 
 static npy_intp
 incr_slot_(double x, double *bins, npy_intp lbins)
@@ -1708,7 +1705,7 @@ static struct PyModuleDef moduledef = {
 
 #if defined(NPY_PY3K)
 #define RETVAL m
-PyObject *PyInit__compiled_base(void)
+PyMODINIT_FUNC PyInit__compiled_base(void)
 #else
 #define RETVAL
 PyMODINIT_FUNC
