@@ -1,36 +1,37 @@
-import numpy
-import types
+from __future__ import division, absolute_import, print_function
+
+import sys
+from functools import reduce
+
+import numpy as np
 from numpy.ma import *
 from numpy.core.numerictypes import float32
 from numpy.ma.core import umath
 from numpy.testing import *
 
-import sys
-if sys.version_info[0] >= 3:
-    from functools import reduce
 
-pi = numpy.pi
+pi = np.pi
 def eq(v, w, msg=''):
     result = allclose(v, w)
     if not result:
-        print """Not eq:%s
+        print("""Not eq:%s
 %s
 ----
-%s""" % (msg, str(v), str(w))
+%s""" % (msg, str(v), str(w)))
     return result
 
 class TestMa(TestCase):
     def setUp (self):
-        x = numpy.array([1., 1., 1., -2., pi / 2.0, 4., 5., -10., 10., 1., 2., 3.])
-        y = numpy.array([5., 0., 3., 2., -1., -4., 0., -10., 10., 1., 0., 3.])
+        x = np.array([1., 1., 1., -2., pi / 2.0, 4., 5., -10., 10., 1., 2., 3.])
+        y = np.array([5., 0., 3., 2., -1., -4., 0., -10., 10., 1., 0., 3.])
         a10 = 10.
         m1 = [1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0]
-        m2 = [0, 0, 1, 0, 0, 1, 1, 0, 0, 0 , 0, 1]
+        m2 = [0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1]
         xm = array(x, mask=m1)
         ym = array(y, mask=m2)
-        z = numpy.array([-.5, 0., .5, .8])
+        z = np.array([-.5, 0., .5, .8])
         zm = array(z, mask=[0, 1, 0, 0])
-        xf = numpy.where(m1, 1e+20, x)
+        xf = np.where(m1, 1e+20, x)
         s = x.shape
         xm.set_fill_value(1e+20)
         self.d = (x, y, a10, m1, m2, xm, ym, z, zm, xf, s)
@@ -43,8 +44,8 @@ class TestMa(TestCase):
         self.assertEqual(shape(xm), s)
         self.assertEqual(xm.shape, s)
         self.assertEqual(xm.dtype, x.dtype)
-        self.assertEqual(xm.size , reduce(lambda x, y:x * y, s))
-        self.assertEqual(count(xm) , len(m1) - reduce(lambda x, y:x + y, m1))
+        self.assertEqual(xm.size, reduce(lambda x, y:x * y, s))
+        self.assertEqual(count(xm), len(m1) - reduce(lambda x, y:x + y, m1))
         self.assertTrue(eq(xm, xf))
         self.assertTrue(eq(filled(xm, 1.e20), xf))
         self.assertTrue(eq(x, xm))
@@ -63,8 +64,8 @@ class TestMa(TestCase):
             self.assertTrue(isMaskedArray(xm))
             self.assertEqual(shape(xm), s)
             self.assertEqual(xm.shape, s)
-            self.assertEqual(xm.size , reduce(lambda x, y:x * y, s))
-            self.assertEqual(count(xm) , len(m1) - reduce(lambda x, y:x + y, m1))
+            self.assertEqual(xm.size, reduce(lambda x, y:x * y, s))
+            self.assertEqual(count(xm), len(m1) - reduce(lambda x, y:x + y, m1))
             self.assertTrue(eq(xm, xf))
             self.assertTrue(eq(filled(xm, 1.e20), xf))
             self.assertTrue(eq(x, xm))
@@ -88,19 +89,13 @@ class TestMa(TestCase):
             self.assertTrue(eq(x + y, xm + ym))
             self.assertTrue(eq(x - y, xm - ym))
             self.assertTrue(eq(x * y, xm * ym))
-            olderr = numpy.seterr(divide='ignore', invalid='ignore')
-            try:
+            with np.errstate(divide='ignore', invalid='ignore'):
                 self.assertTrue(eq(x / y, xm / ym))
-            finally:
-                numpy.seterr(**olderr)
             self.assertTrue(eq(a10 + y, a10 + ym))
             self.assertTrue(eq(a10 - y, a10 - ym))
             self.assertTrue(eq(a10 * y, a10 * ym))
-            olderr = numpy.seterr(divide='ignore', invalid='ignore')
-            try:
+            with np.errstate(divide='ignore', invalid='ignore'):
                 self.assertTrue(eq(a10 / y, a10 / ym))
-            finally:
-                numpy.seterr(**olderr)
             self.assertTrue(eq(x + a10, xm + a10))
             self.assertTrue(eq(x - a10, xm - a10))
             self.assertTrue(eq(x * a10, xm * a10))
@@ -108,18 +103,15 @@ class TestMa(TestCase):
             self.assertTrue(eq(x ** 2, xm ** 2))
             self.assertTrue(eq(abs(x) ** 2.5, abs(xm) ** 2.5))
             self.assertTrue(eq(x ** y, xm ** ym))
-            self.assertTrue(eq(numpy.add(x, y), add(xm, ym)))
-            self.assertTrue(eq(numpy.subtract(x, y), subtract(xm, ym)))
-            self.assertTrue(eq(numpy.multiply(x, y), multiply(xm, ym)))
-            olderr = numpy.seterr(divide='ignore', invalid='ignore')
-            try:
-                self.assertTrue(eq(numpy.divide(x, y), divide(xm, ym)))
-            finally:
-                numpy.seterr(**olderr)
+            self.assertTrue(eq(np.add(x, y), add(xm, ym)))
+            self.assertTrue(eq(np.subtract(x, y), subtract(xm, ym)))
+            self.assertTrue(eq(np.multiply(x, y), multiply(xm, ym)))
+            with np.errstate(divide='ignore', invalid='ignore'):
+                self.assertTrue(eq(np.divide(x, y), divide(xm, ym)))
 
 
     def test_testMixedArithmetic(self):
-        na = numpy.array([1])
+        na = np.array([1])
         ma = array([1])
         self.assertTrue(isinstance(na + ma, MaskedArray))
         self.assertTrue(isinstance(ma + na, MaskedArray))
@@ -127,61 +119,58 @@ class TestMa(TestCase):
     def test_testUfuncs1 (self):
         "Test various functions such as sin, cos."
         (x, y, a10, m1, m2, xm, ym, z, zm, xf, s) = self.d
-        self.assertTrue (eq(numpy.cos(x), cos(xm)))
-        self.assertTrue (eq(numpy.cosh(x), cosh(xm)))
-        self.assertTrue (eq(numpy.sin(x), sin(xm)))
-        self.assertTrue (eq(numpy.sinh(x), sinh(xm)))
-        self.assertTrue (eq(numpy.tan(x), tan(xm)))
-        self.assertTrue (eq(numpy.tanh(x), tanh(xm)))
-        olderr = numpy.seterr(divide='ignore', invalid='ignore')
-        try:
-            self.assertTrue (eq(numpy.sqrt(abs(x)), sqrt(xm)))
-            self.assertTrue (eq(numpy.log(abs(x)), log(xm)))
-            self.assertTrue (eq(numpy.log10(abs(x)), log10(xm)))
-        finally:
-            numpy.seterr(**olderr)
-        self.assertTrue (eq(numpy.exp(x), exp(xm)))
-        self.assertTrue (eq(numpy.arcsin(z), arcsin(zm)))
-        self.assertTrue (eq(numpy.arccos(z), arccos(zm)))
-        self.assertTrue (eq(numpy.arctan(z), arctan(zm)))
-        self.assertTrue (eq(numpy.arctan2(x, y), arctan2(xm, ym)))
-        self.assertTrue (eq(numpy.absolute(x), absolute(xm)))
-        self.assertTrue (eq(numpy.equal(x, y), equal(xm, ym)))
-        self.assertTrue (eq(numpy.not_equal(x, y), not_equal(xm, ym)))
-        self.assertTrue (eq(numpy.less(x, y), less(xm, ym)))
-        self.assertTrue (eq(numpy.greater(x, y), greater(xm, ym)))
-        self.assertTrue (eq(numpy.less_equal(x, y), less_equal(xm, ym)))
-        self.assertTrue (eq(numpy.greater_equal(x, y), greater_equal(xm, ym)))
-        self.assertTrue (eq(numpy.conjugate(x), conjugate(xm)))
-        self.assertTrue (eq(numpy.concatenate((x, y)), concatenate((xm, ym))))
-        self.assertTrue (eq(numpy.concatenate((x, y)), concatenate((x, y))))
-        self.assertTrue (eq(numpy.concatenate((x, y)), concatenate((xm, y))))
-        self.assertTrue (eq(numpy.concatenate((x, y, x)), concatenate((x, ym, x))))
+        self.assertTrue (eq(np.cos(x), cos(xm)))
+        self.assertTrue (eq(np.cosh(x), cosh(xm)))
+        self.assertTrue (eq(np.sin(x), sin(xm)))
+        self.assertTrue (eq(np.sinh(x), sinh(xm)))
+        self.assertTrue (eq(np.tan(x), tan(xm)))
+        self.assertTrue (eq(np.tanh(x), tanh(xm)))
+        with np.errstate(divide='ignore', invalid='ignore'):
+            self.assertTrue (eq(np.sqrt(abs(x)), sqrt(xm)))
+            self.assertTrue (eq(np.log(abs(x)), log(xm)))
+            self.assertTrue (eq(np.log10(abs(x)), log10(xm)))
+        self.assertTrue (eq(np.exp(x), exp(xm)))
+        self.assertTrue (eq(np.arcsin(z), arcsin(zm)))
+        self.assertTrue (eq(np.arccos(z), arccos(zm)))
+        self.assertTrue (eq(np.arctan(z), arctan(zm)))
+        self.assertTrue (eq(np.arctan2(x, y), arctan2(xm, ym)))
+        self.assertTrue (eq(np.absolute(x), absolute(xm)))
+        self.assertTrue (eq(np.equal(x, y), equal(xm, ym)))
+        self.assertTrue (eq(np.not_equal(x, y), not_equal(xm, ym)))
+        self.assertTrue (eq(np.less(x, y), less(xm, ym)))
+        self.assertTrue (eq(np.greater(x, y), greater(xm, ym)))
+        self.assertTrue (eq(np.less_equal(x, y), less_equal(xm, ym)))
+        self.assertTrue (eq(np.greater_equal(x, y), greater_equal(xm, ym)))
+        self.assertTrue (eq(np.conjugate(x), conjugate(xm)))
+        self.assertTrue (eq(np.concatenate((x, y)), concatenate((xm, ym))))
+        self.assertTrue (eq(np.concatenate((x, y)), concatenate((x, y))))
+        self.assertTrue (eq(np.concatenate((x, y)), concatenate((xm, y))))
+        self.assertTrue (eq(np.concatenate((x, y, x)), concatenate((x, ym, x))))
 
     def test_xtestCount (self):
         "Test count"
         ott = array([0., 1., 2., 3.], mask=[1, 0, 0, 0])
         if sys.version_info[0] >= 3:
-            self.assertTrue(isinstance(count(ott), numpy.integer))
+            self.assertTrue(isinstance(count(ott), np.integer))
         else:
-            self.assertTrue(isinstance(count(ott), types.IntType))
+            self.assertTrue(isinstance(count(ott), int))
         self.assertEqual(3, count(ott))
         self.assertEqual(1, count(1))
         self.assertTrue (eq(0, array(1, mask=[1])))
         ott = ott.reshape((2, 2))
-        assert isinstance(count(ott, 0), numpy.ndarray)
+        assert_(isinstance(count(ott, 0), np.ndarray))
         if sys.version_info[0] >= 3:
-            assert isinstance(count(ott), numpy.integer)
+            assert_(isinstance(count(ott), np.integer))
         else:
-            assert isinstance(count(ott), types.IntType)
+            assert_(isinstance(count(ott), int))
         self.assertTrue (eq(3, count(ott)))
-        assert getmask(count(ott, 0)) is nomask
+        assert_(getmask(count(ott, 0)) is nomask)
         self.assertTrue (eq([1, 2], count(ott, 0)))
 
     def test_testMinMax (self):
         "Test minimum and maximum."
         (x, y, a10, m1, m2, xm, ym, z, zm, xf, s) = self.d
-        xr = numpy.ravel(x) #max doesn't work if shaped
+        xr = np.ravel(x) #max doesn't work if shaped
         xmr = ravel(xm)
 
         #true because of careful selection of data
@@ -193,73 +182,73 @@ class TestMa(TestCase):
     def test_testAddSumProd (self):
         "Test add, sum, product."
         (x, y, a10, m1, m2, xm, ym, z, zm, xf, s) = self.d
-        self.assertTrue (eq(numpy.add.reduce(x), add.reduce(x)))
-        self.assertTrue (eq(numpy.add.accumulate(x), add.accumulate(x)))
+        self.assertTrue (eq(np.add.reduce(x), add.reduce(x)))
+        self.assertTrue (eq(np.add.accumulate(x), add.accumulate(x)))
         self.assertTrue (eq(4, sum(array(4), axis=0)))
         self.assertTrue (eq(4, sum(array(4), axis=0)))
-        self.assertTrue (eq(numpy.sum(x, axis=0), sum(x, axis=0)))
-        self.assertTrue (eq(numpy.sum(filled(xm, 0), axis=0), sum(xm, axis=0)))
-        self.assertTrue (eq(numpy.sum(x, 0), sum(x, 0)))
-        self.assertTrue (eq(numpy.product(x, axis=0), product(x, axis=0)))
-        self.assertTrue (eq(numpy.product(x, 0), product(x, 0)))
-        self.assertTrue (eq(numpy.product(filled(xm, 1), axis=0),
+        self.assertTrue (eq(np.sum(x, axis=0), sum(x, axis=0)))
+        self.assertTrue (eq(np.sum(filled(xm, 0), axis=0), sum(xm, axis=0)))
+        self.assertTrue (eq(np.sum(x, 0), sum(x, 0)))
+        self.assertTrue (eq(np.product(x, axis=0), product(x, axis=0)))
+        self.assertTrue (eq(np.product(x, 0), product(x, 0)))
+        self.assertTrue (eq(np.product(filled(xm, 1), axis=0),
                             product(xm, axis=0)))
         if len(s) > 1:
-            self.assertTrue (eq(numpy.concatenate((x, y), 1),
+            self.assertTrue (eq(np.concatenate((x, y), 1),
                                 concatenate((xm, ym), 1)))
-            self.assertTrue (eq(numpy.add.reduce(x, 1), add.reduce(x, 1)))
-            self.assertTrue (eq(numpy.sum(x, 1), sum(x, 1)))
-            self.assertTrue (eq(numpy.product(x, 1), product(x, 1)))
+            self.assertTrue (eq(np.add.reduce(x, 1), add.reduce(x, 1)))
+            self.assertTrue (eq(np.sum(x, 1), sum(x, 1)))
+            self.assertTrue (eq(np.product(x, 1), product(x, 1)))
 
 
     def test_testCI(self):
         "Test of conversions and indexing"
-        x1 = numpy.array([1, 2, 4, 3])
+        x1 = np.array([1, 2, 4, 3])
         x2 = array(x1, mask=[1, 0, 0, 0])
         x3 = array(x1, mask=[0, 1, 0, 1])
         x4 = array(x1)
     # test conversion to strings
         junk, garbage = str(x2), repr(x2)
-        assert eq(numpy.sort(x1), sort(x2, fill_value=0))
+        assert_(eq(np.sort(x1), sort(x2, fill_value=0)))
     # tests of indexing
-        assert type(x2[1]) is type(x1[1])
-        assert x1[1] == x2[1]
-        assert x2[0] is masked
-        assert eq(x1[2], x2[2])
-        assert eq(x1[2:5], x2[2:5])
-        assert eq(x1[:], x2[:])
-        assert eq(x1[1:], x3[1:])
+        assert_(type(x2[1]) is type(x1[1]))
+        assert_(x1[1] == x2[1])
+        assert_(x2[0] is masked)
+        assert_(eq(x1[2], x2[2]))
+        assert_(eq(x1[2:5], x2[2:5]))
+        assert_(eq(x1[:], x2[:]))
+        assert_(eq(x1[1:], x3[1:]))
         x1[2] = 9
         x2[2] = 9
-        assert eq(x1, x2)
+        assert_(eq(x1, x2))
         x1[1:3] = 99
         x2[1:3] = 99
-        assert eq(x1, x2)
+        assert_(eq(x1, x2))
         x2[1] = masked
-        assert eq(x1, x2)
+        assert_(eq(x1, x2))
         x2[1:3] = masked
-        assert eq(x1, x2)
+        assert_(eq(x1, x2))
         x2[:] = x1
         x2[1] = masked
-        assert allequal(getmask(x2), array([0, 1, 0, 0]))
+        assert_(allequal(getmask(x2), array([0, 1, 0, 0])))
         x3[:] = masked_array([1, 2, 3, 4], [0, 1, 1, 0])
-        assert allequal(getmask(x3), array([0, 1, 1, 0]))
+        assert_(allequal(getmask(x3), array([0, 1, 1, 0])))
         x4[:] = masked_array([1, 2, 3, 4], [0, 1, 1, 0])
-        assert allequal(getmask(x4), array([0, 1, 1, 0]))
-        assert allequal(x4, array([1, 2, 3, 4]))
-        x1 = numpy.arange(5) * 1.0
+        assert_(allequal(getmask(x4), array([0, 1, 1, 0])))
+        assert_(allequal(x4, array([1, 2, 3, 4])))
+        x1 = np.arange(5) * 1.0
         x2 = masked_values(x1, 3.0)
-        assert eq(x1, x2)
-        assert allequal(array([0, 0, 0, 1, 0], MaskType), x2.mask)
-        assert eq(3.0, x2.fill_value)
+        assert_(eq(x1, x2))
+        assert_(allequal(array([0, 0, 0, 1, 0], MaskType), x2.mask))
+        assert_(eq(3.0, x2.fill_value))
         x1 = array([1, 'hello', 2, 3], object)
-        x2 = numpy.array([1, 'hello', 2, 3], object)
+        x2 = np.array([1, 'hello', 2, 3], object)
         s1 = x1[1]
         s2 = x2[1]
         self.assertEqual(type(s2), str)
         self.assertEqual(type(s1), str)
         self.assertEqual(s1, s2)
-        assert x1[1:1].shape == (0,)
+        assert_(x1[1:1].shape == (0,))
 
     def test_testCopySize(self):
         "Tests of some subtle points of copying and sizing."
@@ -270,7 +259,7 @@ class TestMa(TestCase):
         m3 = make_mask(m, copy=1)
         self.assertTrue(m is not m3)
 
-        x1 = numpy.arange(5)
+        x1 = np.arange(5)
         y1 = array(x1, mask=m)
         self.assertTrue(y1._data is not x1)
         self.assertTrue(allequal(x1, y1._data))
@@ -323,89 +312,89 @@ class TestMa(TestCase):
     def test_testMaPut(self):
         (x, y, a10, m1, m2, xm, ym, z, zm, xf, s) = self.d
         m = [1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1]
-        i = numpy.nonzero(m)[0]
+        i = np.nonzero(m)[0]
         put(ym, i, zm)
-        assert all(take(ym, i, axis=0) == zm)
+        assert_(all(take(ym, i, axis=0) == zm))
 
     def test_testOddFeatures(self):
         "Test of other odd features"
         x = arange(20); x = x.reshape(4, 5)
         x.flat[5] = 12
-        assert x[1, 0] == 12
+        assert_(x[1, 0] == 12)
         z = x + 10j * x
-        assert eq(z.real, x)
-        assert eq(z.imag, 10 * x)
-        assert eq((z * conjugate(z)).real, 101 * x * x)
+        assert_(eq(z.real, x))
+        assert_(eq(z.imag, 10 * x))
+        assert_(eq((z * conjugate(z)).real, 101 * x * x))
         z.imag[...] = 0.0
 
         x = arange(10)
         x[3] = masked
-        assert str(x[3]) == str(masked)
+        assert_(str(x[3]) == str(masked))
         c = x >= 8
-        assert count(where(c, masked, masked)) == 0
-        assert shape(where(c, masked, masked)) == c.shape
-        z = where(c , x, masked)
-        assert z.dtype is x.dtype
-        assert z[3] is masked
-        assert z[4] is masked
-        assert z[7] is masked
-        assert z[8] is not masked
-        assert z[9] is not masked
-        assert eq(x, z)
-        z = where(c , masked, x)
-        assert z.dtype is x.dtype
-        assert z[3] is masked
-        assert z[4] is not masked
-        assert z[7] is not masked
-        assert z[8] is masked
-        assert z[9] is masked
+        assert_(count(where(c, masked, masked)) == 0)
+        assert_(shape(where(c, masked, masked)) == c.shape)
+        z = where(c, x, masked)
+        assert_(z.dtype is x.dtype)
+        assert_(z[3] is masked)
+        assert_(z[4] is masked)
+        assert_(z[7] is masked)
+        assert_(z[8] is not masked)
+        assert_(z[9] is not masked)
+        assert_(eq(x, z))
+        z = where(c, masked, x)
+        assert_(z.dtype is x.dtype)
+        assert_(z[3] is masked)
+        assert_(z[4] is not masked)
+        assert_(z[7] is not masked)
+        assert_(z[8] is masked)
+        assert_(z[9] is masked)
         z = masked_where(c, x)
-        assert z.dtype is x.dtype
-        assert z[3] is masked
-        assert z[4] is not masked
-        assert z[7] is not masked
-        assert z[8] is masked
-        assert z[9] is masked
-        assert eq(x, z)
+        assert_(z.dtype is x.dtype)
+        assert_(z[3] is masked)
+        assert_(z[4] is not masked)
+        assert_(z[7] is not masked)
+        assert_(z[8] is masked)
+        assert_(z[9] is masked)
+        assert_(eq(x, z))
         x = array([1., 2., 3., 4., 5.])
         c = array([1, 1, 1, 0, 0])
         x[2] = masked
         z = where(c, x, -x)
-        assert eq(z, [1., 2., 0., -4., -5])
+        assert_(eq(z, [1., 2., 0., -4., -5]))
         c[0] = masked
         z = where(c, x, -x)
-        assert eq(z, [1., 2., 0., -4., -5])
-        assert z[0] is masked
-        assert z[1] is not masked
-        assert z[2] is masked
-        assert eq(masked_where(greater(x, 2), x), masked_greater(x, 2))
-        assert eq(masked_where(greater_equal(x, 2), x),
-                  masked_greater_equal(x, 2))
-        assert eq(masked_where(less(x, 2), x), masked_less(x, 2))
-        assert eq(masked_where(less_equal(x, 2), x), masked_less_equal(x, 2))
-        assert eq(masked_where(not_equal(x, 2), x), masked_not_equal(x, 2))
-        assert eq(masked_where(equal(x, 2), x), masked_equal(x, 2))
-        assert eq(masked_where(not_equal(x, 2), x), masked_not_equal(x, 2))
-        assert eq(masked_inside(range(5), 1, 3), [0, 199, 199, 199, 4])
-        assert eq(masked_outside(range(5), 1, 3), [199, 1, 2, 3, 199])
-        assert eq(masked_inside(array(range(5), mask=[1, 0, 0, 0, 0]), 1, 3).mask,
-                  [1, 1, 1, 1, 0])
-        assert eq(masked_outside(array(range(5), mask=[0, 1, 0, 0, 0]), 1, 3).mask,
-                  [1, 1, 0, 0, 1])
-        assert eq(masked_equal(array(range(5), mask=[1, 0, 0, 0, 0]), 2).mask,
-                  [1, 0, 1, 0, 0])
-        assert eq(masked_not_equal(array([2, 2, 1, 2, 1], mask=[1, 0, 0, 0, 0]), 2).mask,
-                  [1, 0, 1, 0, 1])
-        assert eq(masked_where([1, 1, 0, 0, 0], [1, 2, 3, 4, 5]), [99, 99, 3, 4, 5])
+        assert_(eq(z, [1., 2., 0., -4., -5]))
+        assert_(z[0] is masked)
+        assert_(z[1] is not masked)
+        assert_(z[2] is masked)
+        assert_(eq(masked_where(greater(x, 2), x), masked_greater(x, 2)))
+        assert_(eq(masked_where(greater_equal(x, 2), x),
+                  masked_greater_equal(x, 2)))
+        assert_(eq(masked_where(less(x, 2), x), masked_less(x, 2)))
+        assert_(eq(masked_where(less_equal(x, 2), x), masked_less_equal(x, 2)))
+        assert_(eq(masked_where(not_equal(x, 2), x), masked_not_equal(x, 2)))
+        assert_(eq(masked_where(equal(x, 2), x), masked_equal(x, 2)))
+        assert_(eq(masked_where(not_equal(x, 2), x), masked_not_equal(x, 2)))
+        assert_(eq(masked_inside(list(range(5)), 1, 3), [0, 199, 199, 199, 4]))
+        assert_(eq(masked_outside(list(range(5)), 1, 3), [199, 1, 2, 3, 199]))
+        assert_(eq(masked_inside(array(list(range(5)), mask=[1, 0, 0, 0, 0]), 1, 3).mask,
+                  [1, 1, 1, 1, 0]))
+        assert_(eq(masked_outside(array(list(range(5)), mask=[0, 1, 0, 0, 0]), 1, 3).mask,
+                  [1, 1, 0, 0, 1]))
+        assert_(eq(masked_equal(array(list(range(5)), mask=[1, 0, 0, 0, 0]), 2).mask,
+                  [1, 0, 1, 0, 0]))
+        assert_(eq(masked_not_equal(array([2, 2, 1, 2, 1], mask=[1, 0, 0, 0, 0]), 2).mask,
+                  [1, 0, 1, 0, 1]))
+        assert_(eq(masked_where([1, 1, 0, 0, 0], [1, 2, 3, 4, 5]), [99, 99, 3, 4, 5]))
         atest = ones((10, 10, 10), dtype=float32)
         btest = zeros(atest.shape, MaskType)
         ctest = masked_where(btest, atest)
-        assert eq(atest, ctest)
+        assert_(eq(atest, ctest))
         z = choose(c, (-x, x))
-        assert eq(z, [1., 2., 0., -4., -5])
-        assert z[0] is masked
-        assert z[1] is not masked
-        assert z[2] is masked
+        assert_(eq(z, [1., 2., 0., -4., -5]))
+        assert_(z[0] is masked)
+        assert_(z[1] is not masked)
+        assert_(z[2] is masked)
         x = arange(6)
         x[5] = masked
         y = arange(6) * 10
@@ -414,46 +403,46 @@ class TestMa(TestCase):
         cm = c.filled(1)
         z = where(c, x, y)
         zm = where(cm, x, y)
-        assert eq(z, zm)
-        assert getmask(zm) is nomask
-        assert eq(zm, [0, 1, 2, 30, 40, 50])
+        assert_(eq(z, zm))
+        assert_(getmask(zm) is nomask)
+        assert_(eq(zm, [0, 1, 2, 30, 40, 50]))
         z = where(c, masked, 1)
-        assert eq(z, [99, 99, 99, 1, 1, 1])
+        assert_(eq(z, [99, 99, 99, 1, 1, 1]))
         z = where(c, 1, masked)
-        assert eq(z, [99, 1, 1, 99, 99, 99])
+        assert_(eq(z, [99, 1, 1, 99, 99, 99]))
 
-    def test_testMinMax(self):
+    def test_testMinMax2(self):
         "Test of minumum, maximum."
-        assert eq(minimum([1, 2, 3], [4, 0, 9]), [1, 0, 3])
-        assert eq(maximum([1, 2, 3], [4, 0, 9]), [4, 2, 9])
+        assert_(eq(minimum([1, 2, 3], [4, 0, 9]), [1, 0, 3]))
+        assert_(eq(maximum([1, 2, 3], [4, 0, 9]), [4, 2, 9]))
         x = arange(5)
         y = arange(5) - 2
         x[3] = masked
         y[0] = masked
-        assert eq(minimum(x, y), where(less(x, y), x, y))
-        assert eq(maximum(x, y), where(greater(x, y), x, y))
-        assert minimum(x) == 0
-        assert maximum(x) == 4
+        assert_(eq(minimum(x, y), where(less(x, y), x, y)))
+        assert_(eq(maximum(x, y), where(greater(x, y), x, y)))
+        assert_(minimum(x) == 0)
+        assert_(maximum(x) == 4)
 
     def test_testTakeTransposeInnerOuter(self):
         "Test of take, transpose, inner, outer products"
         x = arange(24)
-        y = numpy.arange(24)
+        y = np.arange(24)
         x[5:6] = masked
         x = x.reshape(2, 3, 4)
         y = y.reshape(2, 3, 4)
-        assert eq(numpy.transpose(y, (2, 0, 1)), transpose(x, (2, 0, 1)))
-        assert eq(numpy.take(y, (2, 0, 1), 1), take(x, (2, 0, 1), 1))
-        assert eq(numpy.inner(filled(x, 0), filled(y, 0)),
-                                inner(x, y))
-        assert eq(numpy.outer(filled(x, 0), filled(y, 0)),
-                                outer(x, y))
+        assert_(eq(np.transpose(y, (2, 0, 1)), transpose(x, (2, 0, 1))))
+        assert_(eq(np.take(y, (2, 0, 1), 1), take(x, (2, 0, 1), 1)))
+        assert_(eq(np.inner(filled(x, 0), filled(y, 0)),
+                                inner(x, y)))
+        assert_(eq(np.outer(filled(x, 0), filled(y, 0)),
+                                outer(x, y)))
         y = array(['abc', 1, 'def', 2, 3], object)
         y[2] = masked
         t = take(y, [0, 3, 4])
-        assert t[0] == 'abc'
-        assert t[1] == 2
-        assert t[2] == 3
+        assert_(t[0] == 'abc')
+        assert_(t[1] == 2)
+        assert_(t[2] == 3)
 
     def test_testInplace(self):
         """Test of inplace operations and rich comparisons"""
@@ -463,47 +452,47 @@ class TestMa(TestCase):
         xm = arange(10)
         xm[2] = masked
         x += 1
-        assert eq(x, y + 1)
+        assert_(eq(x, y + 1))
         xm += 1
-        assert eq(x, y + 1)
+        assert_(eq(x, y + 1))
 
         x = arange(10)
         xm = arange(10)
         xm[2] = masked
         x -= 1
-        assert eq(x, y - 1)
+        assert_(eq(x, y - 1))
         xm -= 1
-        assert eq(xm, y - 1)
+        assert_(eq(xm, y - 1))
 
         x = arange(10) * 1.0
         xm = arange(10) * 1.0
         xm[2] = masked
         x *= 2.0
-        assert eq(x, y * 2)
+        assert_(eq(x, y * 2))
         xm *= 2.0
-        assert eq(xm, y * 2)
+        assert_(eq(xm, y * 2))
 
         x = arange(10) * 2
         xm = arange(10)
         xm[2] = masked
-        x /= 2
-        assert eq(x, y)
-        xm /= 2
-        assert eq(x, y)
+        x //= 2
+        assert_(eq(x, y))
+        xm //= 2
+        assert_(eq(x, y))
 
         x = arange(10) * 1.0
         xm = arange(10) * 1.0
         xm[2] = masked
         x /= 2.0
-        assert eq(x, y / 2.0)
+        assert_(eq(x, y / 2.0))
         xm /= arange(10)
-        assert eq(xm, ones((10,)))
+        assert_(eq(xm, ones((10,))))
 
         x = arange(10).astype(float32)
         xm = arange(10)
         xm[2] = masked
         x += 1.
-        assert eq(x, y + 1.)
+        assert_(eq(x, y + 1.))
 
     def test_testPickle(self):
         "Test of pickling"
@@ -513,7 +502,7 @@ class TestMa(TestCase):
         x = x.reshape(4, 3)
         s = pickle.dumps(x)
         y = pickle.loads(s)
-        assert eq(x, y)
+        assert_(eq(x, y))
 
     def test_testMasked(self):
         "Test of masked element"
@@ -556,8 +545,8 @@ class TestMa(TestCase):
         self.assertTrue(allclose(average(x, axis=0, weights=w1), 2.5))
         y = array([arange(6), 2.0 * arange(6)])
         self.assertTrue(allclose(average(y, None),
-                                 numpy.add.reduce(numpy.arange(6)) * 3. / 12.))
-        self.assertTrue(allclose(average(y, axis=0), numpy.arange(6) * 3. / 2.))
+                                 np.add.reduce(np.arange(6)) * 3. / 12.))
+        self.assertTrue(allclose(average(y, axis=0), np.arange(6) * 3. / 2.))
         self.assertTrue(allclose(average(y, axis=1),
                                  [average(x, axis=0), average(x, axis=0) * 2.0]))
         self.assertTrue(allclose(average(y, None, weights=w2), 20. / 6.))
@@ -585,12 +574,12 @@ class TestMa(TestCase):
         a = arange(6)
         b = arange(6) * 3
         r1, w1 = average([[a, b], [b, a]], axis=1, returned=1)
-        self.assertEqual(shape(r1) , shape(w1))
-        self.assertEqual(r1.shape , w1.shape)
+        self.assertEqual(shape(r1), shape(w1))
+        self.assertEqual(r1.shape, w1.shape)
         r2, w2 = average(ones((2, 2, 3)), axis=0, weights=[3, 1], returned=1)
-        self.assertEqual(shape(w2) , shape(r2))
+        self.assertEqual(shape(w2), shape(r2))
         r2, w2 = average(ones((2, 2, 3)), returned=1)
-        self.assertEqual(shape(w2) , shape(r2))
+        self.assertEqual(shape(w2), shape(r2))
         r2, w2 = average(ones((2, 2, 3)), weights=ones((2, 2, 3)), returned=1)
         self.assertTrue(shape(w2) == shape(r2))
         a2d = array([[1, 2], [0, 4]], float)
@@ -616,12 +605,9 @@ class TestMa(TestCase):
     def test_testScalarArithmetic(self):
         xm = array(0, mask=1)
         #TODO FIXME: Find out what the following raises a warning in r8247
-        err_status = numpy.geterr()
-        try:
-            numpy.seterr(divide='ignore')
+        with np.errstate():
+            np.seterr(divide='ignore')
             self.assertTrue((1 / array(0)).mask)
-        finally:
-            numpy.seterr(**err_status)
         self.assertTrue((1 + xm).mask)
         self.assertTrue((-xm).mask)
         self.assertTrue((-xm).mask)
@@ -655,7 +641,7 @@ class TestMa(TestCase):
         self.assertEqual(a.ndim, 1)
 
     def test_testAPI(self):
-        self.assertFalse([m for m in dir(numpy.ndarray)
+        self.assertFalse([m for m in dir(np.ndarray)
                      if m not in dir(MaskedArray) and not m.startswith('_')])
 
     def test_testSingleElementSubscript(self):
@@ -698,18 +684,15 @@ class TestUfuncs(TestCase):
                 uf = getattr(umath, f)
             except AttributeError:
                 uf = getattr(fromnumeric, f)
-            mf = getattr(numpy.ma, f)
+            mf = getattr(np.ma, f)
             args = self.d[:uf.nin]
-            olderr = numpy.geterr()
-            try:
+            with np.errstate():
                 if f in f_invalid_ignore:
-                    numpy.seterr(invalid='ignore')
+                    np.seterr(invalid='ignore')
                 if f in ['arctanh', 'log', 'log10']:
-                    numpy.seterr(divide='ignore')
+                    np.seterr(divide='ignore')
                 ur = uf(*args)
                 mr = mf(*args)
-            finally:
-                numpy.seterr(**olderr)
             self.assertTrue(eq(ur.filled(0), mr.filled(0), f))
             self.assertTrue(eqmask(ur.mask, mr.mask))
 
@@ -739,16 +722,16 @@ class TestUfuncs(TestCase):
 class TestArrayMethods(TestCase):
 
     def setUp(self):
-        x = numpy.array([ 8.375, 7.545, 8.828, 8.5  , 1.757, 5.928,
-                          8.43 , 7.78 , 9.865, 5.878, 8.979, 4.732,
+        x = np.array([ 8.375, 7.545, 8.828, 8.5, 1.757, 5.928,
+                          8.43, 7.78, 9.865, 5.878, 8.979, 4.732,
                           3.012, 6.022, 5.095, 3.116, 5.238, 3.957,
-                          6.04 , 9.63 , 7.712, 3.382, 4.489, 6.479,
+                          6.04, 9.63, 7.712, 3.382, 4.489, 6.479,
                           7.189, 9.645, 5.395, 4.961, 9.894, 2.893,
                           7.357, 9.828, 6.272, 3.758, 6.693, 0.993])
         X = x.reshape(6, 6)
         XX = x.reshape(3, 2, 2, 3)
 
-        m = numpy.array([0, 1, 0, 1, 0, 0,
+        m = np.array([0, 1, 0, 1, 0, 0,
                          1, 0, 1, 1, 0, 1,
                          0, 0, 0, 1, 0, 1,
                          0, 0, 0, 1, 1, 1,
@@ -758,7 +741,7 @@ class TestArrayMethods(TestCase):
         mX = array(data=X, mask=m.reshape(X.shape))
         mXX = array(data=XX, mask=m.reshape(XX.shape))
 
-        m2 = numpy.array([1, 1, 0, 1, 0, 0,
+        m2 = np.array([1, 1, 0, 1, 0, 0,
                           1, 1, 1, 1, 0, 1,
                           0, 0, 1, 1, 0, 1,
                           0, 0, 0, 1, 1, 1,
@@ -788,8 +771,8 @@ class TestArrayMethods(TestCase):
         (x, X, XX, m, mx, mX, mXX,) = self.d
         (n, m) = X.shape
         self.assertEqual(mx.ptp(), mx.compressed().ptp())
-        rows = numpy.zeros(n, numpy.float_)
-        cols = numpy.zeros(m, numpy.float_)
+        rows = np.zeros(n, np.float_)
+        cols = np.zeros(m, np.float_)
         for k in range(m):
             cols[k] = mX[:, k].compressed().ptp()
         for k in range(n):
@@ -829,7 +812,7 @@ class TestArrayMethods(TestCase):
         for k in range(6):
             self.assertTrue(eq(mXvar1[k], mX[k].compressed().var()))
             self.assertTrue(eq(mXvar0[k], mX[:, k].compressed().var()))
-            self.assertTrue(eq(numpy.sqrt(mXvar0[k]),
+            self.assertTrue(eq(np.sqrt(mXvar0[k]),
                                mX[:, k].compressed().std()))
 
 
@@ -855,7 +838,7 @@ def eqmask(m1, m2):
 #""" % (n, t*1000.0, t1/t, t2/t)
 
 #def testta(n, f):
-#    x=numpy.arange(n) + 1.0
+#    x=np.arange(n) + 1.0
 #    tn0 = time.time()
 #    z = f(x)
 #    return time.time() - tn0
