@@ -1,10 +1,12 @@
+from __future__ import division, print_function
+
 from os.path import join
 
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
-    config = Configuration('numarray',parent_package,top_path)
+    config = Configuration('numarray', parent_package, top_path)
 
-    config.add_data_files('numpy/*')
+    config.add_data_files('include/numpy/*')
 
     config.add_extension('_capi',
                          sources=['_capi.c'],
