@@ -1,10 +1,23 @@
+from __future__ import division, absolute_import, print_function
+
 from numpy.testing import TestCase, run_module_suite, assert_,\
-        assert_raises
+        assert_raises, assert_equal
 from numpy import random
 from numpy.compat import asbytes
 import numpy as np
 
 
+class TestBinomial(TestCase):
+    def test_n_zero(self):
+        # Tests the corner case of n == 0 for the binomial distribution.
+        # binomial(0, p) should be zero for any p in [0, 1].
+        # This test addresses issue #3480.
+        zeros = np.zeros(2, dtype='int')
+        for p in [0, .5, 1]:
+            assert_(random.binomial(0, p) == 0)
+            np.testing.assert_array_equal(random.binomial(zeros, p), zeros)
+
+
 class TestMultinomial(TestCase):
     def test_basic(self):
         random.multinomial(100, [0.2, 0.8])
@@ -13,11 +26,25 @@ class TestMultinomial(TestCase):
         random.multinomial(100, [0.2, 0.8, 0.0, 0.0, 0.0])
 
     def test_int_negative_interval(self):
-        assert_( -5 <= random.randint(-5,-1) < -1)
-        x = random.randint(-5,-1,5)
+        assert_( -5 <= random.randint(-5, -1) < -1)
+        x = random.randint(-5, -1, 5)
         assert_(np.all(-5 <= x))
         assert_(np.all(x < -1))
 
+    def test_size(self):
+        # gh-3173
+        p = [0.5, 0.5]
+        assert_equal(np.random.multinomial(1 ,p, np.uint32(1)).shape, (1, 2))
+        assert_equal(np.random.multinomial(1 ,p, np.uint32(1)).shape, (1, 2))
+        assert_equal(np.random.multinomial(1 ,p, np.uint32(1)).shape, (1, 2))
+        assert_equal(np.random.multinomial(1 ,p, [2, 2]).shape, (2, 2, 2))
+        assert_equal(np.random.multinomial(1 ,p, (2, 2)).shape, (2, 2, 2))
+        assert_equal(np.random.multinomial(1 ,p, np.array((2, 2))).shape,
+                     (2, 2, 2))
+
+        assert_raises(TypeError, np.random.multinomial, 1 , p,
+                      np.float(1))
+
 
 class TestSetState(TestCase):
     def setUp(self):
@@ -81,20 +108,20 @@ class TestRandomDist(TestCase):
         actual = np.random.rand(3, 2)
         desired = np.array([[ 0.61879477158567997,  0.59162362775974664],
                          [ 0.88868358904449662,  0.89165480011560816],
-                         [ 0.4575674820298663 ,  0.7781880808593471 ]])
+                         [ 0.4575674820298663,  0.7781880808593471 ]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=15)
 
     def test_randn(self):
         np.random.seed(self.seed)
         actual = np.random.randn(3, 2)
         desired = np.array([[ 1.34016345771863121,  1.73759122771936081],
-                         [ 1.498988344300628  , -0.2286433324536169 ],
-                         [ 2.031033998682787  ,  2.17032494605655257]])
+                         [ 1.498988344300628, -0.2286433324536169 ],
+                         [ 2.031033998682787,  2.17032494605655257]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=15)
 
     def test_randint(self):
         np.random.seed(self.seed)
-        actual = np.random.randint(-99, 99, size=(3,2))
+        actual = np.random.randint(-99, 99, size=(3, 2))
         desired = np.array([[ 31,   3],
                          [-52,  41],
                          [-48, -66]])
@@ -102,7 +129,7 @@ class TestRandomDist(TestCase):
 
     def test_random_integers(self):
         np.random.seed(self.seed)
-        actual = np.random.random_integers(-99, 99, size=(3,2))
+        actual = np.random.random_integers(-99, 99, size=(3, 2))
         desired = np.array([[ 31,   3],
                          [-52,  41],
                          [-48, -66]])
@@ -113,9 +140,91 @@ class TestRandomDist(TestCase):
         actual = np.random.random_sample((3, 2))
         desired = np.array([[ 0.61879477158567997,  0.59162362775974664],
                          [ 0.88868358904449662,  0.89165480011560816],
-                         [ 0.4575674820298663 ,  0.7781880808593471 ]])
+                         [ 0.4575674820298663,  0.7781880808593471 ]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=15)
 
+    def test_choice_uniform_replace(self):
+        np.random.seed(self.seed)
+        actual = np.random.choice(4, 4)
+        desired = np.array([2, 3, 2, 3])
+        np.testing.assert_array_equal(actual, desired)
+
+    def test_choice_nonuniform_replace(self):
+        np.random.seed(self.seed)
+        actual = np.random.choice(4, 4, p=[0.4, 0.4, 0.1, 0.1])
+        desired = np.array([1, 1, 2, 2])
+        np.testing.assert_array_equal(actual, desired)
+
+    def test_choice_uniform_noreplace(self):
+        np.random.seed(self.seed)
+        actual = np.random.choice(4, 3, replace=False)
+        desired = np.array([0, 1, 3])
+        np.testing.assert_array_equal(actual, desired)
+
+    def test_choice_nonuniform_noreplace(self):
+        np.random.seed(self.seed)
+        actual = np.random.choice(4, 3, replace=False,
+                                  p=[0.1, 0.3, 0.5, 0.1])
+        desired = np.array([2, 3, 1])
+        np.testing.assert_array_equal(actual, desired)
+
+    def test_choice_noninteger(self):
+        np.random.seed(self.seed)
+        actual = np.random.choice(['a', 'b', 'c', 'd'], 4)
+        desired = np.array(['c', 'd', 'c', 'd'])
+        np.testing.assert_array_equal(actual, desired)
+
+    def test_choice_exceptions(self):
+        sample = np.random.choice
+        assert_raises(ValueError, sample, -1, 3)
+        assert_raises(ValueError, sample, 3., 3)
+        assert_raises(ValueError, sample, [[1, 2], [3, 4]], 3)
+        assert_raises(ValueError, sample, [], 3)
+        assert_raises(ValueError, sample, [1, 2, 3, 4], 3,
+                                          p=[[0.25, 0.25], [0.25, 0.25]])
+        assert_raises(ValueError, sample, [1, 2], 3, p=[0.4, 0.4, 0.2])
+        assert_raises(ValueError, sample, [1, 2], 3, p=[1.1, -0.1])
+        assert_raises(ValueError, sample, [1, 2], 3, p=[0.4, 0.4])
+        assert_raises(ValueError, sample, [1, 2, 3], 4, replace=False)
+        assert_raises(ValueError, sample, [1, 2, 3], 2, replace=False,
+                                          p=[1, 0, 0])
+
+    def test_choice_return_shape(self):
+        p = [0.1, 0.9]
+        # Check scalar
+        assert_(np.isscalar(np.random.choice(2, replace=True)))
+        assert_(np.isscalar(np.random.choice(2, replace=False)))
+        assert_(np.isscalar(np.random.choice(2, replace=True, p=p)))
+        assert_(np.isscalar(np.random.choice(2, replace=False, p=p)))
+        assert_(np.isscalar(np.random.choice([1, 2], replace=True)))
+        assert_(np.random.choice([None], replace=True) is None)
+        a = np.array([1, 2])
+        arr = np.empty(1, dtype=object)
+        arr[0] = a
+        assert_(np.random.choice(arr, replace=True) is a)
+
+        # Check 0-d array
+        s = tuple()
+        assert_(not np.isscalar(np.random.choice(2, s, replace=True)))
+        assert_(not np.isscalar(np.random.choice(2, s, replace=False)))
+        assert_(not np.isscalar(np.random.choice(2, s, replace=True, p=p)))
+        assert_(not np.isscalar(np.random.choice(2, s, replace=False, p=p)))
+        assert_(not np.isscalar(np.random.choice([1, 2], s, replace=True)))
+        assert_(np.random.choice([None], s, replace=True).ndim == 0)
+        a = np.array([1, 2])
+        arr = np.empty(1, dtype=object)
+        arr[0] = a
+        assert_(np.random.choice(arr, s, replace=True).item() is a)
+
+        # Check multi dimensional array
+        s = (2, 3)
+        p = [0.1, 0.1, 0.1, 0.1, 0.4, 0.2]
+        assert_(np.random.choice(6, s, replace=True).shape, s)
+        assert_(np.random.choice(6, s, replace=False).shape, s)
+        assert_(np.random.choice(6, s, replace=True, p=p).shape, s)
+        assert_(np.random.choice(6, s, replace=False, p=p).shape, s)
+        assert_(np.random.choice(np.arange(6), s, replace=True).shape, s)
+
     def test_bytes(self):
         np.random.seed(self.seed)
         actual = np.random.bytes(10)
@@ -123,12 +232,40 @@ class TestRandomDist(TestCase):
         np.testing.assert_equal(actual, desired)
 
     def test_shuffle(self):
-        np.random.seed(self.seed)
-        alist = [1,2,3,4,5,6,7,8,9,0]
-        np.random.shuffle(alist)
-        actual = alist
-        desired = [0, 1, 9, 6, 2, 4, 5, 8, 7, 3]
-        np.testing.assert_array_equal(actual, desired)
+        # Test lists, arrays, and multidimensional versions of both:
+        for conv in [lambda x: x,
+                     np.asarray,
+                     lambda x: [(i, i) for i in x],
+                     lambda x: np.asarray([(i, i) for i in x])]:
+            np.random.seed(self.seed)
+            alist = conv([1, 2, 3, 4, 5, 6, 7, 8, 9, 0])
+            np.random.shuffle(alist)
+            actual = alist
+            desired = conv([0, 1, 9, 6, 2, 4, 5, 8, 7, 3])
+            np.testing.assert_array_equal(actual, desired)
+
+    def test_shuffle_flexible(self):
+        # gh-4270
+        arr = [(0, 1), (2, 3)]
+        dt = np.dtype([('a', np.int32, 1), ('b', np.int32, 1)])
+        nparr = np.array(arr, dtype=dt)
+        a, b = nparr[0].copy(), nparr[1].copy()
+        for i in range(50):
+            np.random.shuffle(nparr)
+            assert_(a in nparr)
+            assert_(b in nparr)
+
+    def test_shuffle_masked(self):
+        # gh-3263
+        a = np.ma.masked_values(np.reshape(range(20), (5,4)) % 3 - 1, -1)
+        b = np.ma.masked_values(np.arange(20) % 3 - 1, -1)
+        ma = np.ma.count_masked(a)
+        mb = np.ma.count_masked(b)
+        for i in range(50):
+            np.random.shuffle(a)
+            self.assertEqual(ma, np.ma.count_masked(a))
+            np.random.shuffle(b)
+            self.assertEqual(mb, np.ma.count_masked(b))
 
     def test_beta(self):
         np.random.seed(self.seed)
@@ -151,7 +288,7 @@ class TestRandomDist(TestCase):
         actual = np.random.chisquare(50, size=(3, 2))
         desired = np.array([[ 63.87858175501090585,  68.68407748911370447],
                             [ 65.77116116901505904,  47.09686762438974483],
-                            [ 72.3828403199695174 ,  74.18408615260374006]])
+                            [ 72.3828403199695174,  74.18408615260374006]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=13)
 
     def test_dirichlet(self):
@@ -166,6 +303,18 @@ class TestRandomDist(TestCase):
                              [ 0.56974431743975207,  0.43025568256024799]]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=15)
 
+    def test_dirichlet_size(self):
+        # gh-3173
+        p = np.array([51.72840233779265162,  39.74494232180943953])
+        assert_equal(np.random.dirichlet(p, np.uint32(1)).shape, (1, 2))
+        assert_equal(np.random.dirichlet(p, np.uint32(1)).shape, (1, 2))
+        assert_equal(np.random.dirichlet(p, np.uint32(1)).shape, (1, 2))
+        assert_equal(np.random.dirichlet(p, [2, 2]).shape, (2, 2, 2))
+        assert_equal(np.random.dirichlet(p, (2, 2)).shape, (2, 2, 2))
+        assert_equal(np.random.dirichlet(p, np.array((2, 2))).shape, (2, 2, 2))
+
+        assert_raises(TypeError, np.random.dirichlet, p, np.float(1))
+
     def test_exponential(self):
         np.random.seed(self.seed)
         actual = np.random.exponential(1.1234, size=(3, 2))
@@ -202,7 +351,7 @@ class TestRandomDist(TestCase):
         np.random.seed(self.seed)
         actual = np.random.gumbel(loc = .123456789, scale = 2.0, size = (3, 2))
         desired = np.array([[ 0.19591898743416816,  0.34405539668096674],
-                         [-1.4492522252274278 , -1.47374816298446865],
+                         [-1.4492522252274278, -1.47374816298446865],
                          [ 1.10651090478803416, -0.69535848626236174]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=15)
 
@@ -214,6 +363,24 @@ class TestRandomDist(TestCase):
                          [ 9,  9]])
         np.testing.assert_array_equal(actual, desired)
 
+        # Test nbad = 0
+        actual = np.random.hypergeometric(5, 0, 3, size=4)
+        desired = np.array([3, 3, 3, 3])
+        np.testing.assert_array_equal(actual, desired)
+
+        actual = np.random.hypergeometric(15, 0, 12, size=4)
+        desired = np.array([12, 12, 12, 12])
+        np.testing.assert_array_equal(actual, desired)
+
+        # Test ngood = 0
+        actual = np.random.hypergeometric(0, 5, 3, size=4)
+        desired = np.array([0, 0, 0, 0])
+        np.testing.assert_array_equal(actual, desired)
+
+        actual = np.random.hypergeometric(0, 15, 12, size=4)
+        desired = np.array([0, 0, 0, 0])
+        np.testing.assert_array_equal(actual, desired)
+
     def test_laplace(self):
         np.random.seed(self.seed)
         actual = np.random.laplace(loc=.123456789, scale=2.0, size=(3, 2))
@@ -260,7 +427,7 @@ class TestRandomDist(TestCase):
     def test_multivariate_normal(self):
         np.random.seed(self.seed)
         mean= (.123456789, 10)
-        cov = [[1,0],[1,0]]
+        cov = [[1, 0], [1, 0]]
         size = (3, 2)
         actual = np.random.multivariate_normal(mean, cov, size)
         desired = np.array([[[ -1.47027513018564449,  10.                 ],
@@ -300,7 +467,7 @@ class TestRandomDist(TestCase):
         np.random.seed(self.seed)
         actual = np.random.normal(loc = .123456789, scale = 2.0, size = (3, 2))
         desired = np.array([[ 2.80378370443726244,  3.59863924443872163],
-                         [ 3.121433477601256  , -0.33382987590723379],
+                         [ 3.121433477601256, -0.33382987590723379],
                          [ 4.18552478636557357,  4.46410668111310471]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=15)
 
@@ -310,7 +477,13 @@ class TestRandomDist(TestCase):
         desired = np.array([[  2.46852460439034849e+03,   1.41286880810518346e+03],
                          [  5.28287797029485181e+07,   6.57720981047328785e+07],
                          [  1.40840323350391515e+02,   1.98390255135251704e+05]])
-        np.testing.assert_array_almost_equal(actual, desired, decimal=14)
+        # For some reason on 32-bit x86 Ubuntu 12.10 the [1, 0] entry in this
+        # matrix differs by 24 nulps. Discussion:
+        #   http://mail.scipy.org/pipermail/numpy-discussion/2012-September/063801.html
+        # Consensus is that this is probably some gcc quirk that affects
+        # rounding but not in any important way, so we just use a looser
+        # tolerance on this test:
+        np.testing.assert_array_almost_equal_nulp(actual, desired, nulp=30)
 
     def test_poisson(self):
         np.random.seed(self.seed)
@@ -339,7 +512,7 @@ class TestRandomDist(TestCase):
     def test_rayleigh(self):
         np.random.seed(self.seed)
         actual = np.random.rayleigh(scale = 10, size = (3, 2))
-        desired = np.array([[ 13.8882496494248393 ,  13.383318339044731  ],
+        desired = np.array([[ 13.8882496494248393,  13.383318339044731  ],
                          [ 20.95413364294492098,  21.08285015800712614],
                          [ 11.06066537006854311,  17.35468505778271009]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=14)
@@ -356,8 +529,8 @@ class TestRandomDist(TestCase):
         np.random.seed(self.seed)
         actual = np.random.standard_exponential(size = (3, 2))
         desired = np.array([[ 0.96441739162374596,  0.89556604882105506],
-                         [ 2.1953785836319808 ,  2.22243285392490542],
-                         [ 0.6116915921431676 ,  1.50592546727413201]])
+                         [ 2.1953785836319808,  2.22243285392490542],
+                         [ 0.6116915921431676,  1.50592546727413201]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=15)
 
     def test_standard_gamma(self):
@@ -372,8 +545,8 @@ class TestRandomDist(TestCase):
         np.random.seed(self.seed)
         actual = np.random.standard_normal(size = (3, 2))
         desired = np.array([[ 1.34016345771863121,  1.73759122771936081],
-                         [ 1.498988344300628  , -0.2286433324536169 ],
-                         [ 2.031033998682787  ,  2.17032494605655257]])
+                         [ 1.498988344300628, -0.2286433324536169 ],
+                         [ 2.031033998682787,  2.17032494605655257]])
         np.testing.assert_array_almost_equal(actual, desired, decimal=15)
 
     def test_standard_t(self):
